@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.c,v 1.63 2005/06/03 13:42:33 chpe Exp $
+ *  $Id: ephy-embed.c,v 1.67 2005/10/13 03:14:11 philipl Exp $
  */
 
 #include "config.h"
@@ -361,13 +361,30 @@ ephy_embed_base_init (gpointer g_class)
 		g_signal_new ("ge-search-key-press",
 			      EPHY_TYPE_EMBED,
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EphyEmbedIface, context_menu),
+			      G_STRUCT_OFFSET (EphyEmbedIface, search_key_press),
 			      g_signal_accumulator_true_handled, NULL,
 			      ephy_marshal_BOOLEAN__BOXED,
 			      G_TYPE_BOOLEAN,
 			      1,
 			      GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE);
 
+/**
+ * EphyEmbed::close-request
+ * @embed:
+ *
+ * The ::close signal is emitted when the embed request closing.
+ * Return %TRUE to prevent closing. You HAVE to process removal of the embed
+ * as soon as possible after that.
+ **/
+		g_signal_new ("close-request",
+			      EPHY_TYPE_EMBED,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyEmbedIface, close_request),
+			      g_signal_accumulator_true_handled, NULL,
+			      ephy_marshal_BOOLEAN__VOID,
+			      G_TYPE_BOOLEAN,
+			      0);
+
 		initialized = TRUE;
 	}
 }
@@ -700,6 +717,28 @@ ephy_embed_shistory_go_nth (EphyEmbed *embed,
 	iface->shistory_go_nth (embed, nth);
 }
 
+/**
+ * ephy_embed_shistory_copy:
+ * @source: the #EphyEmbed to copy the history from
+ * @dest: the #EphyEmbed to copy the history to
+ * @copy_back: %TRUE to copy the back history
+ * @copy_forward: %TRUE to copy the forward history
+ * @copy_current: %TRUE to set the current page to that in the copied history
+ *
+ * Copy's the back and/or forward history from @source to @dest,
+ * and optionally set @dest to the current page of @source as well.
+ **/
+void
+ephy_embed_shistory_copy (EphyEmbed *source,
+			  EphyEmbed *dest,
+			  gboolean copy_back,
+			  gboolean copy_forward,
+			  gboolean copy_current)
+{
+	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (source);
+	iface->shistory_copy (source, dest, copy_back, copy_forward, copy_current);
+}
+
 /**
  * ephy_embed_get_security_level:
  * @embed: an #EphyEmbed
@@ -732,16 +771,16 @@ ephy_embed_show_page_certificate (EphyEmbed *embed)
 }
 
 /**
- * ephy_embed_activate:
+ * ephy_embed_close:
  * @embed: an #EphyEmbed
  *
- * Gives focus to @embed (i.e., Mozilla).
+ * Closes the @embed
  **/
 void
-ephy_embed_activate (EphyEmbed *embed)
+ephy_embed_close (EphyEmbed *embed)
 {
 	EphyEmbedIface *iface = EPHY_EMBED_GET_IFACE (embed);
-	iface->activate (embed);
+	iface->close (embed);
 }
 
 /**
