@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.h,v 1.56 2005/07/29 20:12:55 chpe Exp $
+ *  $Id: ephy-embed.h,v 1.59 2005/10/13 03:14:11 philipl Exp $
  */
 
 #ifndef EPHY_EMBED_H
@@ -51,7 +51,8 @@ typedef enum
 	EPHY_EMBED_STATE_IS_REQUEST	= 1 << 5,
 	EPHY_EMBED_STATE_IS_DOCUMENT	= 1 << 6,
 	EPHY_EMBED_STATE_IS_NETWORK	= 1 << 7,
-	EPHY_EMBED_STATE_IS_WINDOW	= 1 << 8
+	EPHY_EMBED_STATE_IS_WINDOW	= 1 << 8,
+	EPHY_EMBED_STATE_RESTORING	= 1 << 9
 } EphyEmbedNetState;
 
 typedef enum
@@ -148,6 +149,7 @@ struct _EphyEmbedIface
 					 EphyEmbed *new_embed);
 	gboolean (* search_key_press)	(EphyEmbed *embed,
 					 GdkEventKey *event);
+	gboolean (* close_request)	(EphyEmbed *embed);
 
 	/* Methods  */
 	void		   (* load_url)			(EphyEmbed *embed,
@@ -177,6 +179,11 @@ struct _EphyEmbedIface
 	int		   (* shistory_get_pos)		(EphyEmbed *embed);
 	void		   (* shistory_go_nth)		(EphyEmbed *embed,
 							 int nth);
+	void		   (* shistory_copy)		(EphyEmbed *source,
+							 EphyEmbed *dest,
+							 gboolean copy_back,
+							 gboolean copy_forward,
+							 gboolean copy_current);
 	void		   (* get_security_level)	(EphyEmbed *embed,
 						  	 EphyEmbedSecurityLevel *level,
 						  	 char **description);
@@ -196,6 +203,7 @@ struct _EphyEmbedIface
 							 EphyEmbedPrintPreviewNavType type,
 							 int page);
 	gboolean	   (* has_modified_forms)	(EphyEmbed *embed);
+	void		   (* close)			(EphyEmbed *embed);
 };
 
 GType 		  ephy_embed_net_state_get_type		(void);
@@ -256,6 +264,12 @@ int		  ephy_embed_shistory_get_pos		(EphyEmbed *embed);
 void		  ephy_embed_shistory_go_nth		(EphyEmbed *embed,
 							 int nth);
 
+void		  ephy_embed_shistory_copy		(EphyEmbed *source,
+							 EphyEmbed *dest,
+							 gboolean copy_back,
+							 gboolean copy_forward,
+							 gboolean copy_current);
+
 void		  ephy_embed_get_security_level		(EphyEmbed *embed,
 							 EphyEmbedSecurityLevel *level,
 						 	 char **description);
@@ -289,6 +303,8 @@ void		  ephy_embed_print_preview_navigate	(EphyEmbed *embed,
 							 int page);
 
 /* Misc. utility */
+void		  ephy_embed_close			(EphyEmbed *embed);
+
 gboolean	  ephy_embed_has_modified_forms		(EphyEmbed *embed);
 
 G_END_DECLS
