@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history.h,v 1.13 2005/08/28 13:54:16 crispin Exp $
+ *  $Id: ephy-history.h,v 1.14 2005/09/05 16:41:12 chpe Exp $
  */
 
 #ifndef EPHY_HISTORY_H
@@ -48,7 +48,8 @@ enum
 	EPHY_NODE_PAGE_PROP_HOST_ID = 7,
 	EPHY_NODE_PAGE_PROP_PRIORITY = 8,
 	EPHY_NODE_PAGE_PROP_ICON = 9,
-	EPHY_NODE_HOST_PROP_ZOOM = 10
+	EPHY_NODE_HOST_PROP_ZOOM = 10,
+	EPHY_NODE_PAGE_PROP_GECKO_FLAGS = 11
 };
 
 struct _EphyHistory
