@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyAboutModule.h,v 1.1 2005/07/17 16:06:04 chpe Exp $
+ *  $Id: EphyAboutModule.h,v 1.2 2005/10/12 19:33:48 chpe Exp $
  */
  
 #ifndef EPHY_PROTOCOL_HANDLER_H
@@ -31,13 +31,19 @@
 { 0xa9aea13e, 0x21de, 0x4be8, \
   { 0xa0, 0x7e, 0xa0, 0x5f, 0x11, 0x65, 0x8c, 0x55 } }
 
+#ifdef HAVE_GECKO_1_8
 #define EPHY_ABOUT_NETERROR_CONTRACTID	NS_ABOUT_MODULE_CONTRACTID_PREFIX "neterror"
 #define EPHY_ABOUT_NETERROR_CLASSNAME	"Epiphany about:neterror module"
+#endif
+
 #define EPHY_ABOUT_EPIPHANY_CONTRACTID	NS_ABOUT_MODULE_CONTRACTID_PREFIX "epiphany"
 #define EPHY_ABOUT_EPIPHANY_CLASSNAME	"Epiphany about:epiphany module"
+#define EPHY_ABOUT_RECOVER_CONTRACTID	NS_ABOUT_MODULE_CONTRACTID_PREFIX "recover"
+#define EPHY_ABOUT_RECOVER_CLASSNAME	"Epiphany about:recover module"
 
 class nsIChannel;
 class nsIOutputStream;
+class nsIInputStreamChannel;
 class nsIURI;
 
 class EphyAboutModule : public nsIAboutModule
@@ -51,9 +57,13 @@ class EphyAboutModule : public nsIAboutModule
 
   private:
 	nsresult Redirect(const nsACString&, nsIChannel**);
-	nsresult ParseErrorURL(const char*, nsACString&, nsACString&, nsACString&, nsACString&);
+	nsresult ParseURL(const char*, nsACString&, nsACString&, nsACString&, nsACString&, nsACString&);
+#ifdef HAVE_GECKO_1_8
 	nsresult GetErrorMessage(nsIURI*, const char*, char**, char**, char**, char**);
 	nsresult CreateErrorPage(nsIURI*, nsIChannel**);
+#endif
+	nsresult CreateRecoverPage(nsIURI*, nsIChannel**);
+	nsresult WritePage(nsIURI*, nsIURI*, const nsACString&, const char*, const char*, const char*, const char*, const char*, const char*, nsIInputStreamChannel**);
 	nsresult Write(nsIOutputStream*, const char*);
 };
 
