@@ -15,18 +15,21 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyUtils.h,v 1.6.2.1 2005/06/02 20:53:01 chpe Exp $
+ *  $Id: EphyUtils.h,v 1.9 2005/07/23 21:31:19 chpe Exp $
  */
 
 #ifndef EPHY_UTILS_H
 #define EPHY_UTILS_H
 
-#include <nsIIOService.h>
-#include <nsIURI.h>
-#include <nsIDOMWindow.h>
+#include <nsError.h>
 #include <gtk/gtkwidget.h>
 
+class nsAString;
+class nsACString;
+class nsIDOMWindow;
+class nsIIOService;
 class nsIPrintSettings;
+class nsIURI;
 struct _EmbedPrintInfo;
 
 namespace EphyUtils
@@ -47,7 +50,7 @@ namespace EphyUtils
 
 	GtkWidget      *FindGtkParent		(nsIDOMWindow *aDOMWindow);
 
-	nsresult        CollatePrintSettings	(const _EmbedPrintInfo *info,
+	nsresult        CollatePrintSettings	(_EmbedPrintInfo *info,
 						 nsIPrintSettings *settings,
 						 gboolean preview);
 	PRBool          StringEquals            (nsAString &s1, 
