@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.60.2.1 2006/01/13 11:46:45 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.61 2005/09/26 19:22:19 jframeau Exp $
  */
 
 #include "mozilla-config.h"
@@ -266,7 +266,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	rv = node->GetNodeType(&type);
 	if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-	PRBool has_background = PR_FALSE;
+	PRBool has_image = PR_FALSE;
 
 	nsCOMPtr<nsIDOMHTMLElement> element = do_QueryInterface(node);
 	if ((nsIDOMNode::ELEMENT_NODE == type) && element)
@@ -280,8 +280,6 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 		if (g_ascii_strcasecmp (tag.get(), "img") == 0)
 		{
-			info->context |= EPHY_EMBED_CONTEXT_IMAGE;
-
 			nsEmbedString img;
 			nsCOMPtr <nsIDOMHTMLImageElement> image = 
 						do_QueryInterface(node, &rv);
@@ -289,7 +287,10 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 			rv = image->GetSrc (img);
 			if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
+
 			SetStringProperty ("image", img);
+			info->context |= EPHY_EMBED_CONTEXT_IMAGE;
+			has_image = PR_TRUE;
 		}
 		else if (g_ascii_strcasecmp (tag.get(), "area") == 0)
 		{
@@ -343,9 +344,9 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					rv = img->GetSrc (imgSrc);
 					if (NS_FAILED(rv)) continue;
 
-					info->context |= EPHY_EMBED_CONTEXT_IMAGE;
-
 					SetStringProperty ("image", imgSrc);
+					info->context |= EPHY_EMBED_CONTEXT_IMAGE;
+					has_image = PR_TRUE;
 
 					break;
 				}
@@ -374,8 +375,6 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			// MIME types are always lower case
 			if (g_str_has_prefix (cValue.get(), "image/"))
 			{
-				info->context |= EPHY_EMBED_CONTEXT_IMAGE;
-				
 				nsEmbedString img;
 				
 				rv = object->GetData (img);
@@ -386,6 +385,8 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
                                 if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 
 				SetStringProperty ("image", cImg.get());
+				info->context |= EPHY_EMBED_CONTEXT_IMAGE;
+				has_image = PR_TRUE;
 			}
 			else
 			{
@@ -415,10 +416,9 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					if (NS_FAILED (rv))
 						return NS_ERROR_FAILURE;
 
-					SetStringProperty ("background_image",
-							   bgimg.get());
-
-					has_background = PR_TRUE;
+					SetStringProperty ("image", bgimg.get());
+					info->context |= EPHY_EMBED_CONTEXT_IMAGE;
+					has_image = PR_TRUE;
 				}
 			}
  		}
@@ -587,7 +587,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				info->context |= EPHY_EMBED_CONTEXT_INPUT;
 			}
 
-			if (!has_background)
+			if (!has_image)
 			{
 				nsEmbedString cssurl;
 				rv = GetCSSBackground (node, cssurl);
@@ -598,10 +598,9 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
                                         rv = ResolveBaseURL (cssurl, bgimg);
                                         if (NS_FAILED (rv))
                                                 return NS_ERROR_FAILURE;
-					SetStringProperty ("background_image",
-						           bgimg.get());
-
-					has_background = PR_TRUE;
+					SetStringProperty ("image", bgimg.get());
+					info->context |= EPHY_EMBED_CONTEXT_IMAGE;
+					has_image = PR_TRUE;
 				}
 			}
 		}
@@ -1043,7 +1042,10 @@ EventContext::CheckKeyPress (nsIDOMKeyEvent *aEvent)
 	rv = aEvent->GetTarget (getter_AddRefs (target));
 	NS_ENSURE_SUCCESS (rv, retval);
 
-	nsCOMPtr<nsIDOMHTMLInputElement> inputElement (do_QueryInterface (target));
+	nsCOMPtr<nsIDOMHTMLElement> element (do_QueryInterface (target, &rv));
+	NS_ENSURE_SUCCESS (rv, retval);
+
+	nsCOMPtr<nsIDOMHTMLInputElement> inputElement (do_QueryInterface (element));
 	if (inputElement)
 	{
 		nsEmbedString type;
@@ -1061,34 +1063,27 @@ EventContext::CheckKeyPress (nsIDOMKeyEvent *aEvent)
 	nsCOMPtr<nsIDOMHTMLSelectElement> selectElement;
 	nsCOMPtr<nsIDOMHTMLIsIndexElement> indexElement;
 
-	if ((textArea = do_QueryInterface (target)) ||
-	    (selectElement = do_QueryInterface (target)) ||
-	    (indexElement = do_QueryInterface (target))) return retval;
+	if ((textArea = do_QueryInterface (element)) ||
+	    (selectElement = do_QueryInterface (element)) ||
+	    (indexElement = do_QueryInterface (element))) return retval;
 
 	/* check for design mode */
-	nsCOMPtr<nsIDOMNode> node (do_QueryInterface (target, &rv));
-	NS_ENSURE_SUCCESS (rv, PR_FALSE);
-
 	nsCOMPtr<nsIDOMDocument> doc;
-	rv = node->GetOwnerDocument (getter_AddRefs (doc));
+	rv = element->GetOwnerDocument (getter_AddRefs (doc));
 	NS_ENSURE_SUCCESS (rv, retval);
 
-	nsCOMPtr<nsIDOMNSHTMLDocument> htmlDoc (do_QueryInterface (doc));
-	if (htmlDoc)
-	{
-		nsEmbedString uDesign;
-		rv = htmlDoc->GetDesignMode (uDesign);
-		NS_ENSURE_SUCCESS (rv, retval);
+	nsCOMPtr<nsIDOMNSHTMLDocument> htmlDoc (do_QueryInterface (doc, &rv));
+	/* FIXME: return PR_TRUE here ? */
+	if (NS_FAILED (rv)) return retval; /* it's okay not to be a HTML document */
 
-		nsEmbedCString design;
-		NS_UTF16ToCString (uDesign, NS_CSTRING_ENCODING_UTF8, design);
+	nsEmbedString uDesign;
+	rv = htmlDoc->GetDesignMode (uDesign);
+	NS_ENSURE_SUCCESS (rv, retval);
 
-		retval = g_ascii_strcasecmp (design.get(), "on") != 0;
-	}
-	else
-	{
-		retval = PR_TRUE;
-	}
+	nsEmbedCString design;
+	NS_UTF16ToCString (uDesign, NS_CSTRING_ENCODING_UTF8, design);
+
+	retval = g_ascii_strcasecmp (design.get(), "on") != 0;
 
 	return retval;
 }
