@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.cpp,v 1.52.4.2 2006/02/02 22:28:04 chpe Exp $
+ *  $Id: FilePicker.cpp,v 1.54 2005/09/28 13:31:22 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -25,7 +25,6 @@
 
 #include "FilePicker.h"
 #include "EphyUtils.h"
-#include "AutoJSContextStack.h"
 
 #include <nsCOMPtr.h>
 #undef MOZILLA_INTERNAL_API
@@ -128,6 +127,10 @@ NS_IMETHODIMP GFilePicker::Init(nsIDOMWindowInternal *parent, const PRUnichar *t
 	{
 		gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (mDialog), TRUE);
 	}
+	if (mMode == nsIFilePicker::modeSave)
+	{
+		gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (mDialog), TRUE);
+	}
 
 	g_object_add_weak_pointer (G_OBJECT (mDialog), (gpointer *) &mDialog);
 
@@ -427,11 +430,6 @@ NS_IMETHODIMP GFilePicker::GetFiles(nsISimpleEnumerator * *aFiles)
 /* short show (); */
 NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 {
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
 	LOG ("GFilePicker::Show");
 
 	gtk_window_set_modal (GTK_WINDOW (mDialog), TRUE);
@@ -468,7 +466,7 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 	}
 	while (response == GTK_RESPONSE_ACCEPT &&
 	       mMode == nsIFilePicker::modeSave &&
-	       !ephy_gui_confirm_overwrite_file (GTK_WIDGET (mDialog), filename));
+	       !ephy_gui_check_location_writable (GTK_WIDGET (mDialog), filename));
 
 	gtk_widget_hide (GTK_WIDGET (mDialog));
 
