@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.cpp,v 1.38.2.2 2006/02/02 20:43:24 chpe Exp $
+ *  $Id: GlobalHistory.cpp,v 1.39 2005/09/05 16:41:12 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -32,9 +32,6 @@
 #include <nsEmbedString.h>
 #define MOZILLA_INTERNAL_API 1
 
-#define MAX_TITLE_LENGTH	2048
-#define MAX_URL_LENGTH		16384
-
 NS_IMPL_ISUPPORTS1 (MozGlobalHistory, nsIGlobalHistory2)
 
 MozGlobalHistory::MozGlobalHistory ()
@@ -94,8 +91,6 @@ NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aT
 	rv = aURI->GetSpec(spec);
 	NS_ENSURE_TRUE (NS_SUCCEEDED(rv) && spec.Length(), rv);
 
-	if (spec.Length () > MAX_URL_LENGTH) return NS_OK;
-
 	ephy_history_add_page (mGlobalHistory, spec.get());
 	
 	return NS_OK;
@@ -106,13 +101,9 @@ NS_IMETHODIMP MozGlobalHistory::IsVisited(nsIURI *aURI, PRBool *_retval)
 {
 	NS_ENSURE_ARG (aURI);
 
-	*_retval = PR_FALSE;
-
 	nsEmbedCString spec;
 	aURI->GetSpec(spec);
 
-	if (spec.Length () > MAX_URL_LENGTH) return NS_OK;
-	
 	*_retval = ephy_history_is_page_visited (mGlobalHistory, spec.get());
 	
 	return NS_OK;
@@ -123,26 +114,14 @@ NS_IMETHODIMP MozGlobalHistory::SetPageTitle(nsIURI *aURI, const nsAString & aTi
 {
 	NS_ENSURE_ARG (aURI);
 
+	nsEmbedCString title;
+	NS_UTF16ToCString (nsEmbedString (aTitle),
+			   NS_CSTRING_ENCODING_UTF8, title);
+
 	nsEmbedCString spec;
 	aURI->GetSpec(spec);
-
-	if (spec.Length () > MAX_URL_LENGTH) return NS_OK;
-
-	nsEmbedString uTitle (aTitle);
-
-	/* This depends on the assumption that 
-	 * typeof(PRUnichar) == typeof (gunichar2) == uint16,
-	 * which should be pretty safe.
-	 */
-	glong n_read = 0, n_written = 0;
-	char *converted = g_utf16_to_utf8 ((gunichar2*) uTitle.get(), MAX_TITLE_LENGTH,
-					&n_read, &n_written, NULL);
-	/* FIXME loop from the end while !g_unichar_isspace (char)? */
-	if (converted == NULL) return NS_OK;
-
-	ephy_history_set_page_title (mGlobalHistory, spec.get(), converted);
-
-	g_free (converted);
+	
+	ephy_history_set_page_title (mGlobalHistory, spec.get(), title.get());
 	
 	return NS_OK;
 }
@@ -150,17 +129,46 @@ NS_IMETHODIMP MozGlobalHistory::SetPageTitle(nsIURI *aURI, const nsAString & aTi
 #ifdef HAVE_GECKO_1_9
 /* unsigned long getURIGeckoFlags(in nsIURI aURI); */
 NS_IMETHODIMP
-GlobalHistory::GetURIGeckoFlags(nsIURI *aURI, PRUint32* aFlags)
+MozGlobalHistory::GetURIGeckoFlags(nsIURI *aURI, PRUint32* aFlags)
 {
-#error Implement me!
-	return NS_ERROR_NOT_IMPLEMENTED:
+	nsEmbedCString spec;
+	aURI->GetSpec(spec);
+
+	EphyNode *page = ephy_history_get_page (mGlobalHistory, spec.get());
+
+	GValue value = { 0, };
+	if (page != NULL &&
+	    ephy_node_get_property (page, EPHY_NODE_PAGE_PROP_GECKO_FLAGS, &value))
+	{
+		*aFlags = (PRUint32) (gulong) g_value_get_long (&value);
+		g_value_unset (&value);
+
+		return NS_OK;
+	}
+
+	return NS_ERROR_FAILURE;
 }
 
 /* void setURIGeckoFlags(in nsIURI aURI, in unsigned long aFlags); */
 NS_IMETHODIMP
-GlobalHistory::SetURIGeckoFlags(nsIURI *aURI, PRUint32 aFlags)
+MozGlobalHistory::SetURIGeckoFlags(nsIURI *aURI, PRUint32 aFlags)
 {
-#error Implement me!
-	return NS_ERROR_NOT_IMPLEMENTED:
+	nsEmbedCString spec;
+	aURI->GetSpec(spec);
+
+	EphyNode *page = ephy_history_get_page (mGlobalHistory, spec.get());
+	if (page != NULL)
+	{
+		GValue value = { 0, };
+
+		g_value_init (&value, G_TYPE_LONG);
+		g_value_set_long (&value, (long) (gulong) aFlags);
+		ephy_node_set_property (page, EPHY_NODE_PAGE_PROP_GECKO_FLAGS, &value);
+		g_value_unset (&value);
+
+		return NS_OK;
+	}
+
+	return NS_ERROR_FAILURE;
 }
 #endif /* HAVE_GECKO_1_9 */
