@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GtkNSSDialogs.cpp,v 1.32.4.3 2006/02/02 22:28:04 chpe Exp $
+ *  $Id: GtkNSSDialogs.cpp,v 1.33 2005/09/15 12:29:03 chpe Exp $
  */
 
 /*
@@ -31,7 +31,6 @@
 #include "config.h"
 
 #include "EphyUtils.h"
-#include "AutoJSContextStack.h"
 
 #include <nsCOMPtr.h>
 #include <nsMemory.h>
@@ -208,11 +207,6 @@ display_cert_warning_box (nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label, *checkbox, *vbox, *button;
 	int res;
 
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
 	/* NOTE: Due to a mozilla bug [https://bugzilla.mozilla.org/show_bug.cgi?id=306288],
 	 * we will always end up without a parent!
 	 */
@@ -543,11 +537,6 @@ GtkNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *label;
 	char *msg, *primary;
 
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -760,11 +749,6 @@ GtkNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *progress;
 	char *msg;
 
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -882,11 +866,6 @@ GtkNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *hbox, *label, *entry, *vbox;
 	char *msg;
 
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
 	nsCOMPtr<nsIDOMWindow> parent = do_GetInterface (ctx);
 	GtkWindow *gparent = GTK_WINDOW (EphyUtils::FindGtkParent (parent));
 
@@ -1118,11 +1097,10 @@ fill_cert_chain_tree (GtkTreeView *treeview, nsIArray *certChain)
 	}
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));
 
-	/* And select the last entry, and scroll the view so it's visible */
+	/* And select the last entry */
 	GtkTreeSelection *select = gtk_tree_view_get_selection (treeview);
 	GtkTreePath *path = gtk_tree_model_get_path (model, &parent);
 	gtk_tree_selection_select_path (select, path);
-	gtk_tree_view_scroll_to_cell (treeview, path, NULL, TRUE, 0.5, 0.0);
 	gtk_tree_path_free (path);
 
 	return TRUE; 
@@ -1316,15 +1294,11 @@ GtkNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx,
 	GtkWidget *dialog, *widget;
 	GladeXML *gxml;
 	nsEmbedString value;
+	nsresult rv;
 	PRUint32 verifystate, count;
 	PRUnichar ** usage;
 	GtkSizeGroup * sizegroup;
 
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
 	gxml = ephy_glade_widget_new (ephy_file ("certificate-dialogs.glade"),
 				      "viewcert_dialog",
 				      &dialog, NULL, NULL);
