@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed-single.cpp,v 1.115.4.1 2006/02/10 13:40:39 chpe Exp $
+ *  $Id: mozilla-embed-single.cpp,v 1.119 2005/10/22 22:11:05 jframeau Exp $
  */
 
 #include "mozilla-config.h"
@@ -33,7 +33,6 @@
 #include "glib.h"
 #include "ephy-debug.h"
 #include "gtkmozembed.h"
-#include "gtkmozembed_internal.h"
 #include "mozilla-embed.h"
 #include "ephy-file-helpers.h"
 #include "mozilla-notifiers.h"
@@ -89,10 +88,6 @@
 #include <nsIIDNService.h>
 #endif /* ALLOW_PRIVATE_API */
 
-#if defined(HAVE_MOZILLA_TOOLKIT) && defined(HAVE_GECKO_1_8)
-#include "EphyDirectoryProvider.h"
-#endif
-
 #include <stdlib.h>
 
 #define MOZILLA_PROFILE_DIR  "/mozilla"
@@ -110,6 +105,14 @@ struct MozillaEmbedSinglePrivate
 	GtkWidget *theme_window;
 
 	EphySingle *mSingleObserver;
+
+	guint online : 1;
+};
+
+enum
+{
+	PROP_0,
+	PROP_NETWORK_STATUS
 };
 
 static void mozilla_embed_single_class_init	(MozillaEmbedSingleClass *klass);
@@ -361,8 +364,11 @@ mozilla_init_plugin_path ()
 	user_path = g_getenv ("MOZ_PLUGIN_PATH");
 	new_path = g_strconcat (user_path ? user_path : "",
 				user_path ? ":" : "",
-				MOZILLA_PREFIX "/lib/mozilla/plugins:"
-				MOZILLA_HOME "/plugins",
+				MOZILLA_PREFIX "/lib/mozilla/plugins"
+				":" MOZILLA_HOME "/plugins",
+#ifdef HAVE_PRIVATE_PLUGINS
+				":" PLUGINDIR,
+#endif
 				NULL);
 
 	g_setenv ("MOZ_PLUGIN_PATH", new_path, TRUE);
@@ -495,14 +501,6 @@ init_services (MozillaEmbedSingle *single)
 	/* Set mozilla binary path */
 	gtk_moz_embed_set_comp_path (MOZILLA_HOME);
 
-#if defined(HAVE_MOZILLA_TOOLKIT) && defined(HAVE_GECKO_1_8)
-
-        nsCOMPtr<nsIDirectoryServiceProvider> dp = new EphyDirectoryProvider ();
-        if (!dp) return FALSE;
-
-        gtk_moz_embed_set_directory_service_provider (dp);
-#endif
-
 	/* Fire up the beast */
 	gtk_moz_embed_push_startup ();
 
@@ -636,18 +634,23 @@ impl_clear_auth_cache (EphyEmbedSingle *shell)
 }
 
 static void
-impl_set_offline_mode (EphyEmbedSingle *shell,
-		       gboolean offline)
+impl_set_network_status (EphyEmbedSingle *single,
+			 gboolean online)
 {
 	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID);
 	if (!io) return;
 
-	io->SetOffline(offline);
+	io->SetOffline (!online);
 }
 
 static gboolean
-impl_get_offline_mode (EphyEmbedSingle *shell)
+impl_get_network_status (EphyEmbedSingle *esingle)
 {
+	MozillaEmbedSingle *single = MOZILLA_EMBED_SINGLE (esingle);
+	MozillaEmbedSinglePrivate *priv = single->priv;
+
+	NS_ENSURE_TRUE (priv->mSingleObserver, TRUE);
+
 	nsCOMPtr<nsIIOService> io = do_GetService(NS_IOSERVICE_CONTRACTID);
 	if (!io) return FALSE; /* no way to check the state, assume offline */
 
@@ -656,7 +659,12 @@ impl_get_offline_mode (EphyEmbedSingle *shell)
 	rv = io->GetOffline(&isOffline);
 	NS_ENSURE_SUCCESS (rv, FALSE);
 
-	return isOffline;
+	PRBool isOnline = !isOffline;
+	PRBool reallyOnline = priv->mSingleObserver->IsOnline ();
+
+	g_return_val_if_fail (reallyOnline == isOnline, TRUE);
+
+	return !isOffline;
 }
 
 static GList *
@@ -802,11 +810,18 @@ impl_list_passwords (EphyPasswordManager *manager)
 		NS_UTF16ToCString (unicodeName,
 				   NS_CSTRING_ENCODING_UTF8, userName);
 
+		rv = nsPassword->GetPassword (unicodeName);
+		if (NS_FAILED (rv)) continue;
+
+		nsEmbedCString userPassword;
+		NS_UTF16ToCString (unicodeName,
+				   NS_CSTRING_ENCODING_UTF8, userPassword);
+
 		EphyPasswordInfo *p = g_new0 (EphyPasswordInfo, 1);
 
 		p->host = g_strdup (host.get());
 		p->username = g_strdup (userName.get());
-		p->password = NULL;
+		p->password = g_strdup (userPassword.get());
 
 		passwords = g_list_prepend (passwords, p);
 	}
@@ -994,6 +1009,22 @@ impl_open_window (EphyEmbedSingle *single,
 	return EphyUtils::FindEmbed (newWindow);
 }
 
+static void
+mozilla_embed_single_get_property (GObject *object,
+				guint prop_id,
+				GValue *value,
+				GParamSpec *pspec)
+{
+	EphyEmbedSingle *single = EPHY_EMBED_SINGLE (object);
+
+	switch (prop_id)
+	{
+		case PROP_NETWORK_STATUS:
+			g_value_set_boolean (value, ephy_embed_single_get_network_status (single));
+			break;
+	}
+}
+
 static void
 mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 {
@@ -1003,6 +1034,9 @@ mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 
 	object_class->dispose = mozilla_embed_single_dispose;
 	object_class->finalize = mozilla_embed_single_finalize;
+	object_class->get_property = mozilla_embed_single_get_property;
+
+	g_object_class_override_property (object_class, PROP_NETWORK_STATUS, "network-status");
 
 	g_type_class_add_private (object_class, sizeof (MozillaEmbedSinglePrivate));
 }
@@ -1012,8 +1046,8 @@ ephy_embed_single_iface_init (EphyEmbedSingleIface *iface)
 {
 	iface->clear_cache = impl_clear_cache;
 	iface->clear_auth_cache = impl_clear_auth_cache;
-	iface->set_offline_mode = impl_set_offline_mode;
-	iface->get_offline_mode = impl_get_offline_mode;
+	iface->set_network_status = impl_set_network_status;
+	iface->get_network_status = impl_get_network_status;
 	iface->get_font_list = impl_get_font_list;
 	iface->open_window = impl_open_window;
 }
