@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mozilla-embed.cpp,v 1.119.2.3 2005/10/14 18:45:32 chpe Exp $
+ *  $Id: mozilla-embed.cpp,v 1.123 2005/10/14 18:45:21 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -42,6 +42,7 @@
 #include <nsMemory.h>
 #include <nsIURI.h>
 #include <nsIRequest.h>
+#include <nsIWebNavigation.h>
 #include <nsIWebProgressListener.h>
 #include <nsGfxCIID.h>
 
@@ -721,6 +722,20 @@ impl_shistory_go_nth (EphyEmbed *embed,
 	mpriv->browser->GoToHistoryIndex (nth);
 }
 
+static void
+impl_shistory_copy (EphyEmbed *source,
+		    EphyEmbed *dest,
+		    gboolean copy_back,
+		    gboolean copy_forward,
+		    gboolean copy_current)
+{
+	MozillaEmbedPrivate *spriv = MOZILLA_EMBED(source)->priv;
+	MozillaEmbedPrivate *dpriv = MOZILLA_EMBED(dest)->priv;
+
+	spriv->browser->CopySHistory(dpriv->browser, copy_back,
+				     copy_forward, copy_current);
+}
+
 static void
 impl_get_security_level (EphyEmbed *embed,
                          EphyEmbedSecurityLevel *level,
@@ -1170,6 +1185,7 @@ ephy_embed_iface_init (EphyEmbedIface *iface)
 	iface->shistory_get_nth = impl_shistory_get_nth;
 	iface->shistory_get_pos = impl_shistory_get_pos;
 	iface->shistory_go_nth = impl_shistory_go_nth;
+	iface->shistory_copy = impl_shistory_copy;
 	iface->get_security_level = impl_get_security_level;
 	iface->show_page_certificate = impl_show_page_certificate;
 	iface->close = impl_close;
