@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: print-dialog.c,v 1.42.4.3 2005/11/27 22:11:06 chpe Exp $
+ *  $Id: print-dialog.c,v 1.44 2005/10/02 18:09:38 chpe Exp $
  */
 
 #include "config.h"
@@ -357,7 +357,7 @@ void
 ephy_print_setup_dialog_help_button_cb (GtkWidget *widget,
 					 EphyDialog *dialog)
 {
-	ephy_gui_help (gtk_widget_get_toplevel (widget), "epiphany", "using-print-setup");
+	ephy_gui_help (GTK_WINDOW (dialog), "epiphany", "using-print-setup");
 }
 
 static GtkWidget *
