@@ -21,7 +21,7 @@
 
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 
-   $Id: eel-gconf-extensions.c,v 1.13 2004/11/30 23:35:15 chpe Exp $
+   $Id: eel-gconf-extensions.c,v 1.14 2005/02/27 18:50:45 chpe Exp $
 */
 
 #include "config.h"
@@ -670,47 +670,19 @@ eel_gconf_get_float (const char *key)
 static char *
 tilde_compress (const char *path)
 {
-	const char *home_dir = g_get_home_dir();
-	int         home_dir_l = strlen (home_dir);
-	int         ntilde = 0;
-	const char *scan;
-	int         path_l, result_l;
-	char       *result, *scan2;
+	const char *home;
 
-	if (path == NULL)
-		return NULL;
+	if (path == NULL) return NULL;
 
-	path_l = strlen (path);
-	for (scan = path; scan != NULL; scan++) {
-		if (path_l - (scan - path) < home_dir_l)
-			break;
-		if (strncmp (scan, home_dir, home_dir_l) == 0)
-			ntilde++;
+	home = g_get_home_dir ();
+	if (home == NULL) return g_strdup (path);
+
+	if (g_str_has_prefix (path, home))
+	{
+		return g_strconcat ("~", path + strlen (home), NULL);
 	}
-	
-	if (ntilde == 0)
-		return g_strdup (path);
-	
-	result_l = strlen (path) + ntilde - (ntilde * home_dir_l);
-	result = g_new (char, result_l + 1);
 
-	for (scan = path, scan2 = result; scan != NULL; scan2++) {
-		if (path_l - (scan - path) < home_dir_l) {
-			strcpy (scan2, scan);
-			scan2 += strlen (scan);
-			break;
-		}
-		if (strncmp (scan, home_dir, home_dir_l) == 0) {
-			*scan2 = '~';
-			scan += home_dir_l;
-		} else {
-			*scan2 = *scan;
-			scan++;
-		} 
-	}
-	*scan2 = 0;
-
-	return result;
+	return g_strdup (path);
 }
 
 void
