@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-module.c,v 1.4 2005/01/28 22:24:03 chpe Exp $
+ *  $Id: ephy-module.c,v 1.6 2005/10/21 14:53:20 chpe Exp $
  */
 
 #include "config.h"
@@ -42,6 +42,7 @@ struct _EphyModule
 
 	char *path;
 	GType type;
+	guint resident : 1;
 };
 
 typedef GType (*EphyModuleRegisterFunc) (GTypeModule *);
@@ -84,10 +85,34 @@ ephy_module_load (GTypeModule *gmodule)
 {
 	EphyModule *module = EPHY_MODULE (gmodule);
 	EphyModuleRegisterFunc register_func;
+	gboolean is_absolute;
 
 	LOG ("Loading %s", module->path);
 
-	module->library = g_module_open (module->path, 0);
+	is_absolute = g_path_is_absolute (module->path);
+
+	if (module->library == NULL && ! is_absolute)
+	{
+		char *path = g_build_filename (EXTENSIONS_DIR, module->path, NULL);
+
+		module->library = g_module_open (path, 0);
+
+		g_free (path);
+	}
+
+	if (module->library == NULL && ! is_absolute)
+	{
+		char *path = g_build_filename (ephy_dot_dir(), "extensions", module->path, NULL);
+		
+		module->library = g_module_open (path, 0);
+
+		g_free (path);
+	}
+
+	if (module->library == NULL)
+	{
+		module->library = g_module_open (module->path, 0);
+	}
 
 	if (module->library == NULL)
 	{
@@ -115,6 +140,11 @@ ephy_module_load (GTypeModule *gmodule)
 		return FALSE;
 	}
 
+	if (module->resident)
+	{
+		g_module_make_resident (module->library);
+	}
+
 	return TRUE;
 }
 
@@ -185,7 +215,8 @@ ephy_module_class_init (EphyModuleClass *class)
 }
 
 EphyModule *
-ephy_module_new (const char *path)
+ephy_module_new (const char *path,
+		 gboolean resident)
 {
 	EphyModule *result;
 
@@ -198,6 +229,7 @@ ephy_module_new (const char *path)
 
 	g_type_module_set_name (G_TYPE_MODULE (result), path);
 	result->path = g_strdup (path);
+	result->resident = resident != FALSE;
 
 	return result;
 }
