@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-db.c,v 1.20 2005/01/08 19:41:46 chpe Exp $
+ *  $Id: ephy-node-db.c,v 1.24 2005/07/03 21:38:54 jframeau Exp $
  */
 
 #include "config.h"
@@ -32,10 +32,6 @@
 #include <unistd.h>
 #include <sys/stat.h>
 
-static void ephy_node_db_class_init (EphyNodeDbClass *klass);
-static void ephy_node_db_init (EphyNodeDb *node);
-static void ephy_node_db_finalize (GObject *object);
-
 /* FIXME I want to find a better way to deal with "root" nodes */
 #define RESERVED_IDS 30
 
@@ -58,60 +54,15 @@ struct _EphyNodeDbPrivate
 	GPtrArray *id_to_node;
 };
 
-static GHashTable *ephy_node_databases = NULL;
-
 static GObjectClass *parent_class = NULL;
 
-GType
-ephy_node_db_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (EphyNodeDbClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_node_db_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyNodeDb),
-			0,
-			(GInstanceInitFunc) ephy_node_db_init
-		};
-
-		type = g_type_register_static (G_TYPE_OBJECT,
-					       "EphyNodeDb",
-					       &our_info, 0);
-	}
-
-	return type;
-}
-
-static void
-ephy_node_db_set_name (EphyNodeDb *db, const char *name)
-{
-	db->priv->name = g_strdup (name);
-
-	if (ephy_node_databases == NULL)
-	{
-		ephy_node_databases = g_hash_table_new (g_str_hash, g_str_equal);
-	}
-
-	g_hash_table_insert (ephy_node_databases, db->priv->name, db);
-}
-
 static void
 ephy_node_db_get_property (GObject *object,
                            guint prop_id,
                            GValue *value,
                            GParamSpec *pspec)
 {
-	EphyNodeDb *db;
-
-	db = EPHY_NODE_DB (object);
+	EphyNodeDb *db = EPHY_NODE_DB (object);
 
 	switch (prop_id)
 	{
@@ -130,14 +81,12 @@ ephy_node_db_set_property (GObject *object,
                            const GValue *value,
                            GParamSpec *pspec)
 {
-	EphyNodeDb *db;
-
-	db = EPHY_NODE_DB (object);
+	EphyNodeDb *db = EPHY_NODE_DB (object);
 
 	switch (prop_id)
 	{
 		case PROP_NAME:
-			ephy_node_db_set_name (db, g_value_get_string (value));
+			db->priv->name = g_value_dup_string (value);
 			break;
 		case PROP_IMMUTABLE:
 			ephy_node_db_set_immutable (db, g_value_get_boolean (value));
@@ -145,36 +94,6 @@ ephy_node_db_set_property (GObject *object,
 	}
 }
 
-static void
-ephy_node_db_class_init (EphyNodeDbClass *klass)
-{
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-	parent_class = g_type_class_peek_parent (klass);
-
-	object_class->finalize = ephy_node_db_finalize;
-        object_class->set_property = ephy_node_db_set_property;
-        object_class->get_property = ephy_node_db_get_property;
-
-	g_object_class_install_property (object_class,
-                                         PROP_NAME,
-                                         g_param_spec_string  ("name",
-                                                               "Name",
-                                                               "Name",
-                                                               NULL,
-                                                               G_PARAM_READWRITE));
-
-	g_object_class_install_property (object_class,
-                                         PROP_IMMUTABLE,
-                                         g_param_spec_boolean  ("immutable",
-								"Immutable",
-								"Immutable",
-								FALSE,
-								G_PARAM_READWRITE));
-
-	g_type_class_add_private (object_class, sizeof (EphyNodeDbPrivate));
-}
-
 static void
 ephy_node_db_init (EphyNodeDb *db)
 {
@@ -192,12 +111,6 @@ ephy_node_db_finalize (GObject *object)
 {
 	EphyNodeDb *db = EPHY_NODE_DB (object);
 
-	g_hash_table_remove (ephy_node_databases, db->priv->name);
-	if (g_hash_table_size (ephy_node_databases) == 0)
-	{
-		g_hash_table_destroy (ephy_node_databases);
-	}
-
 	g_ptr_array_free (db->priv->id_to_node, TRUE);
 
 	g_free (db->priv->name);
@@ -205,24 +118,6 @@ ephy_node_db_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-/**
- * ephy_node_db_get_by_name:
- * @name: the name of the desired #EphyNodeDb
- *
- * Returns the #EphyNodeDb named @name, or %NULL if no such database exists.
- *
- * Return value: an #EphyNodeDb
- **/
-EphyNodeDb *
-ephy_node_db_get_by_name (const char *name)
-{
-	EphyNodeDb *ret;
-
-	ret = g_hash_table_lookup (ephy_node_databases, name);
-
-	return ret;
-}
-
 /**
  * ephy_node_db_new:
  * @name: the name of the new #EphyNodeDb
@@ -251,7 +146,7 @@ node_from_id_real (EphyNodeDb *db, guint id)
 	EphyNode *ret = NULL;
 
 	if (id < db->priv->id_to_node->len)
-		ret = g_ptr_array_index (db->priv->id_to_node, id);;
+		ret = g_ptr_array_index (db->priv->id_to_node, id);
 
 	return ret;
 }
@@ -380,7 +275,7 @@ ephy_node_db_load_from_file (EphyNodeDb *db,
 	gboolean was_immutable;
 	int ret;
 
-	LOG ("ephy_node_db_load_from_file %s", xml_file)
+	LOG ("ephy_node_db_load_from_file %s", xml_file);
 
 	START_PROFILER ("loading node db")
 
@@ -408,7 +303,7 @@ ephy_node_db_load_from_file (EphyNodeDb *db,
 		name = xmlTextReaderConstName (reader);
 		type = xmlTextReaderNodeType (reader);
 
-		if (xmlStrEqual (name, "node")
+		if (xmlStrEqual (name, (const xmlChar *)"node")
 		    && type == XML_READER_TYPE_ELEMENT)
 		{
 			xmlNodePtr subtree;
@@ -429,7 +324,7 @@ ephy_node_db_load_from_file (EphyNodeDb *db,
 			xmlChar *version;
 
 			/* check version info */
-			version = xmlTextReaderGetAttribute (reader, "version");
+			version = xmlTextReaderGetAttribute (reader, (const xmlChar *)"version");
 			if (xmlStrEqual (version, xml_version) == FALSE)
 			{
 				success = FALSE;
@@ -468,12 +363,12 @@ ephy_node_db_write_to_xml_valist (EphyNodeDb *db,
 	EphyNode *node;
 	int ret;
 
-	LOG ("Saving node db to %s", filename)
+	LOG ("Saving node db to %s", filename);
 
 	START_PROFILER ("Saving node db")
 
 	/* FIXME: do we want to turn compression on ? */
-	writer = xmlNewTextWriterFilename (filename, 0);
+	writer = xmlNewTextWriterFilename ((const char *)filename, 0);
 	if (writer == NULL)
 	{
 		return -1;
@@ -482,7 +377,7 @@ ephy_node_db_write_to_xml_valist (EphyNodeDb *db,
 	ret = xmlTextWriterSetIndent (writer, 1);
 	if (ret < 0) goto out;
 
-	ret = xmlTextWriterSetIndentString (writer, "  ");
+	ret = xmlTextWriterSetIndentString (writer, (const xmlChar *)"  ");
 	if (ret < 0) goto out;
 
 	ret = xmlTextWriterStartDocument (writer, "1.0", NULL, NULL);
@@ -491,7 +386,7 @@ ephy_node_db_write_to_xml_valist (EphyNodeDb *db,
 	ret = xmlTextWriterStartElement (writer, root);
 	if (ret < 0) goto out;
 
-	ret = xmlTextWriterWriteAttribute (writer, "version", version);
+	ret = xmlTextWriterWriteAttribute (writer, (const xmlChar *)"version", version);
 	if (ret < 0) goto out;
 
 	if (comment != NULL)
@@ -504,24 +399,21 @@ ephy_node_db_write_to_xml_valist (EphyNodeDb *db,
 	while (node != NULL)
 	{
 		GPtrArray *children;
-		guint n_exceptions, i;
-		GSList *exceptions = NULL;
+		EphyNodeFilterFunc filter;
+		gpointer user_data;
+		int i;
 
-		n_exceptions = va_arg (argptr, guint);
-		for (i=0; i < n_exceptions; i++)
-		{
-			exceptions = g_slist_prepend (exceptions,
-						      va_arg (argptr, EphyNode *));
-		}
+		filter = va_arg (argptr, EphyNodeFilterFunc);
+		user_data = va_arg (argptr, gpointer);
 
 		children = ephy_node_get_children (node);
-		for (i=0; i < children->len; i++)
+		for (i = 0; i < children->len; i++)
 		{
 			EphyNode *kid;
 		
 			kid = g_ptr_array_index (children, i);
 		
-			if (g_slist_find (exceptions, kid) == NULL)
+			if (!filter || filter (kid, user_data))
 			{
 				ret = ephy_node_write_to_xml (kid, writer);
 				if (ret < 0) break;
@@ -529,8 +421,6 @@ ephy_node_db_write_to_xml_valist (EphyNodeDb *db,
 		}
 		if (ret < 0) break;
 
-		g_slist_free (exceptions);
-
 		node = va_arg (argptr, EphyNode *);
 	}
 	if (ret < 0) goto out;
@@ -594,12 +484,12 @@ ephy_node_db_write_to_xml_safe (EphyNodeDb *db,
 	int ret = 0;
 	char *tmp_file;
 
-	tmp_file = g_strconcat (filename, ".tmp", NULL);
+	tmp_file = g_strconcat ((const gchar *)filename, ".tmp", NULL);
 
 	va_start (argptr, node);
  
 	ret = ephy_node_db_write_to_xml_valist
-		(db, tmp_file, root, version, comment, node, argptr);
+		(db, (const xmlChar *)tmp_file, root, version, comment, node, argptr);
 
 	va_end (argptr);
 
@@ -609,7 +499,7 @@ ephy_node_db_write_to_xml_safe (EphyNodeDb *db,
 		goto failed;
 	}
 
-	if (ephy_file_switch_temp_file (filename, tmp_file) == FALSE)
+	if (ephy_file_switch_temp_file ((const char *)filename, tmp_file) == FALSE)
 	{
 		ret = -1;
 	}
@@ -619,3 +509,62 @@ failed:
 
 	return ret;
 }
+
+static void
+ephy_node_db_class_init (EphyNodeDbClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = ephy_node_db_finalize;
+	object_class->set_property = ephy_node_db_set_property;
+	object_class->get_property = ephy_node_db_get_property;
+
+	g_object_class_install_property (object_class,
+					PROP_NAME,
+					g_param_spec_string  ("name",
+							      "Name",
+							      "Name",
+							      NULL,
+							      G_PARAM_READWRITE |
+							      G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					PROP_IMMUTABLE,
+					g_param_spec_boolean ("immutable",
+							      "Immutable",
+							      "Immutable",
+							      FALSE,
+							      G_PARAM_READWRITE));
+
+	g_type_class_add_private (object_class, sizeof (EphyNodeDbPrivate));
+}
+
+GType
+ephy_node_db_get_type (void)
+{
+	static GType type = 0;
+
+	if (G_UNLIKELY (type == 0))
+	{
+		static const GTypeInfo our_info =
+		{
+			sizeof (EphyNodeDbClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) ephy_node_db_class_init,
+			NULL,
+			NULL,
+			sizeof (EphyNodeDb),
+			0,
+			(GInstanceInitFunc) ephy_node_db_init
+		};
+
+		type = g_type_register_static (G_TYPE_OBJECT,
+					       "EphyNodeDb",
+					       &our_info, 0);
+	}
+
+	return type;
+}
