@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-prefs.h,v 1.31 2005/01/30 15:20:35 chpe Exp $
+ *  $Id: ephy-prefs.h,v 1.32 2005/04/19 09:37:12 chpe Exp $
  */
 
 #ifndef EPHY_PREFS_H
@@ -61,6 +61,8 @@ G_BEGIN_DECLS
 /* System prefs */
 #define CONF_DESKTOP_FTP_HANDLER	"/desktop/gnome/url-handlers/ftp/command"
 #define CONF_DESKTOP_TOOLBAR_STYLE	"/desktop/gnome/interface/toolbar_style"
+#define CONF_DESKTOP_BG_PICTURE		"/desktop/gnome/background/picture_filename"
+#define CONF_DESKTOP_BG_TYPE		"/desktop/gnome/background/picture_options"
 
 G_END_DECLS
 
