@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: ephy-find-toolbar.c,v 1.17.2.4 2006/02/05 14:10:15 chpe Exp $
+ *  $Id: ephy-find-toolbar.c,v 1.18 2005/10/01 07:38:05 chpe Exp $
  */
 
 #include "config.h"
@@ -40,6 +40,8 @@
 #include <gtk/gtkmain.h>
 #include <string.h>
 
+#undef FIND_WHILE_TYPING_IN_EMBED
+
 #define EPHY_FIND_TOOLBAR_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object),EPHY_TYPE_FIND_TOOLBAR, EphyFindToolbarPrivate))
 
 struct _EphyFindToolbarPrivate
@@ -54,6 +56,7 @@ struct _EphyFindToolbarPrivate
 	guint preedit_changed : 1;
 	guint prevent_activate : 1;
 	guint activated : 1;
+	guint explicit_focus : 1;
 	guint links_only : 1;
 	guint typing_ahead : 1;
 };
@@ -118,6 +121,42 @@ tab_content_changed_cb (EphyEmbed *embed,
 
 #ifdef HAVE_TYPEAHEADFIND
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+
+/* Cut and paste from gtkwindow.c */
+static void
+send_focus_change (GtkWidget *widget,
+		   gboolean   in)
+{
+	GdkEvent *event;
+
+	event = gdk_event_new (GDK_FOCUS_CHANGE);
+
+	g_object_ref (widget);
+   
+	if (in)
+	{
+		GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
+	}
+	else
+	{
+		GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
+	}
+
+	event->focus_change.type = GDK_FOCUS_CHANGE;
+	event->focus_change.window = (GdkWindow *) g_object_ref (widget->window);
+	event->focus_change.in = in;
+  
+	gtk_widget_event (widget, event);
+  
+	g_object_notify (G_OBJECT (widget), "has_focus");
+
+	g_object_unref (widget);
+	gdk_event_free (event);
+}
+
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
+
 /* Code adapted from gtktreeview.c:gtk_tree_view_key_press() and
  * gtk_tree_view_real_start_interactive_seach()
  */
@@ -128,6 +167,12 @@ tab_search_key_press_cb (EphyEmbed *embed,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 	GtkWidget *widget = (GtkWidget *) toolbar;
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+	GtkEntry *entry = (GtkEntry *) priv->entry;
+	GdkWindow *event_window;
+	gboolean retval = FALSE;
+	guint oldhash, newhash;
+#endif
 
 	g_return_val_if_fail (event != NULL, FALSE);
 
@@ -150,8 +195,58 @@ tab_search_key_press_cb (EphyEmbed *embed,
 	}
 
 	return FALSE;
+
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+	/* don't do anything if the find toolbar is hidden */
+	if (GTK_WIDGET_VISIBLE (widget) == FALSE ||
+	    event->keyval == GDK_Return ||
+	    event->keyval == GDK_KP_Enter ||
+	    event->keyval == GDK_ISO_Enter)
+	{
+		return FALSE;
+	}
+
+	oldhash = g_str_hash (gtk_entry_get_text (entry));
+
+	event_window = event->window;
+	event->window = priv->entry->window;
+
+	/* Send the event to the window.  If the preedit_changed signal is emitted
+	* during this event, we will set priv->imcontext_changed  */
+	priv->preedit_changed = priv->activated = FALSE;
+	priv->prevent_activate = TRUE;
+	retval = gtk_widget_event (priv->entry, (GdkEvent*) event);
+	priv->prevent_activate = FALSE;
+
+	/* restore event window, else gdk_event_free below will crash */
+	event->window = event_window;
+
+	newhash = g_str_hash (gtk_entry_get_text (entry));
+
+	/* FIXME: is this correct? */
+	return retval && (oldhash != newhash || priv->preedit_changed) && !priv->activated;
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
 }
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+
+static gboolean
+tab_dom_mouse_click_cb (EphyEmbed *embed,
+			gpointer event,
+			EphyFindToolbar *toolbar)
+{
+	EphyFindToolbarPrivate *priv = toolbar->priv;
+
+	if (!priv->explicit_focus)
+	{
+		ephy_find_toolbar_close (toolbar);
+	}
+
+	return FALSE;
+}
+
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
+
 #endif /* HAVE_TYPEAHEADFIND */
 
 static void
@@ -191,6 +286,15 @@ entry_changed_cb (GtkEntry *entry,
 	set_controls (toolbar, found, found);
 }
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+static void
+entry_preedit_changed_cb (GtkIMContext *context,
+			  EphyFindToolbar *toolbar)
+{
+	toolbar->priv->preedit_changed = TRUE;
+}
+#endif
+
 static gboolean
 entry_key_press_event_cb (GtkEntry *entry,
 			  GdkEventKey *event,
@@ -202,7 +306,7 @@ entry_key_press_event_cb (GtkEntry *entry,
 	/* Hide the toolbar when ESC is pressed */
 	if ((event->state & mask) == 0 && event->keyval == GDK_Escape)
 	{
-		ephy_find_toolbar_request_close (toolbar);
+		g_signal_emit (toolbar, signals[CLOSE], 0);
 		handled = TRUE;
 	}
 	else if ((event->state & mask) == GDK_CONTROL_MASK &&
@@ -220,6 +324,11 @@ entry_activate_cb (GtkWidget *entry,
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+	priv->activated = TRUE;
+	if (priv->prevent_activate) return;
+#endif
+
 	if (priv->typing_ahead)
 	{
 		ephy_embed_find_activate_link (get_find (toolbar), 0);
@@ -244,9 +353,24 @@ set_focus_cb (EphyWindow *window,
 	}
 
 	/* if widget == toolbar, the new focus widget is in the toolbar */
-	if (widget != wtoolbar)
+	if (widget == wtoolbar)
 	{
-		ephy_find_toolbar_request_close (toolbar);
+		priv->explicit_focus = TRUE;
+	}
+	else if (priv->explicit_focus)
+	{
+		g_signal_emit (toolbar, signals[CLOSE], 0);
+	}
+}
+
+static void
+sync_print_preview_mode (EphyWindow *window,
+			 GParamSpec *pspec,
+			 EphyFindToolbar *toolbar)
+{
+	if (ephy_window_get_is_print_preview (window) && GTK_WIDGET_VISIBLE (GTK_WIDGET (toolbar)))
+	{
+		ephy_find_toolbar_close (toolbar);
 	}
 }
 
@@ -257,6 +381,10 @@ ephy_find_toolbar_set_window (EphyFindToolbar *toolbar,
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
 	priv->window = window;
+
+	sync_print_preview_mode (window, NULL, toolbar);
+	g_signal_connect (window, "notify::print-preview-mode",
+			  G_CALLBACK (sync_print_preview_mode), toolbar);
 }
 
 static void
@@ -283,6 +411,7 @@ ephy_find_toolbar_grab_focus (GtkWidget *widget)
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
 	gtk_widget_grab_focus (GTK_WIDGET (priv->entry));
+	g_return_if_fail (priv->explicit_focus);
 }
 
 static void
@@ -347,6 +476,10 @@ ephy_find_toolbar_init (EphyFindToolbar *toolbar)
 			  G_CALLBACK (entry_key_press_event_cb), toolbar);
 	g_signal_connect_after (priv->entry, "changed",
 				G_CALLBACK (entry_changed_cb), toolbar);
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+	g_signal_connect (GTK_ENTRY (priv->entry)->im_context, "preedit-changed",
+			  G_CALLBACK (entry_preedit_changed_cb), toolbar);
+#endif
 	g_signal_connect (priv->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), toolbar);
 	g_signal_connect_swapped (priv->next, "clicked",
@@ -412,6 +545,7 @@ ephy_find_toolbar_class_init (EphyFindToolbarClass *klass)
 
 	klass->next = ephy_find_toolbar_find_next;
 	klass->previous = ephy_find_toolbar_find_previous;
+	klass->close = ephy_find_toolbar_close;
 
 	signals[NEXT] =
 		g_signal_new ("next",
@@ -516,7 +650,6 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 	if (embed != NULL)
 	{
 		set_controls (toolbar, TRUE, TRUE);
-
 		g_signal_connect_object (embed, "ge-content-change",
 					 G_CALLBACK (tab_content_changed_cb),
 					 toolbar, G_CONNECT_AFTER);
@@ -524,6 +657,11 @@ ephy_find_toolbar_set_embed (EphyFindToolbar *toolbar,
 		g_signal_connect_object (embed, "ge-search-key-press",
 					 G_CALLBACK (tab_search_key_press_cb),
 					 toolbar, 0);
+#ifdef FIND_WHILE_TYPING_IN_EMBED
+		g_signal_connect_object (embed, "dom-mouse-click",
+					 G_CALLBACK (tab_dom_mouse_click_cb),
+					 toolbar, 0);
+#endif /* FIND_WHILE_TYPING_IN_EMBED */
 #endif /* HAVE_TYPEAHEADFIND */
 
 		if (priv->find != NULL)
@@ -565,6 +703,7 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	priv->typing_ahead = typing_ahead;
 	priv->links_only = links_only;
+	priv->explicit_focus = FALSE;
 
 	if (clear_search)
 	{
@@ -577,7 +716,19 @@ ephy_find_toolbar_open (EphyFindToolbar *toolbar,
 
 	gtk_widget_show (GTK_WIDGET (toolbar));
 
+#if defined(HAVE_TYPEAHEADFIND) && defined(FIND_WHILE_TYPING_IN_EMBED)
+	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
+
+	send_focus_change (priv->entry, TRUE);
+
+	if (strlen(gtk_entry_get_text (GTK_ENTRY (priv->entry))) != 0)
+	{
+		entry_changed_cb (GTK_ENTRY (priv->entry), toolbar);
+	}
+#else
 	gtk_widget_grab_focus (GTK_WIDGET (toolbar));
+#endif
+
 }
 
 void
@@ -585,14 +736,13 @@ ephy_find_toolbar_close (EphyFindToolbar *toolbar)
 {
 	EphyFindToolbarPrivate *priv = toolbar->priv;
 
+	g_return_if_fail (priv->embed != NULL);
+	
 	gtk_widget_hide (GTK_WIDGET (toolbar));
-}
 
-void
-ephy_find_toolbar_request_close (EphyFindToolbar *toolbar)
-{
-	if (GTK_WIDGET_VISIBLE (GTK_WIDGET (toolbar)))
-	{
-		g_signal_emit (toolbar, signals[CLOSE], 0);
-	}
+	/* first unset explicit_focus, else we get infinite recursion */
+	priv->explicit_focus = FALSE;
+#ifndef FIND_WHILE_TYPING_IN_EMBED
+	gtk_widget_grab_focus (GTK_WIDGET (priv->embed));
+#endif
 }
