@@ -16,20 +16,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-fullscreen-popup.c,v 1.4 2005/01/26 15:06:42 chpe Exp $
+ *  $Id: ephy-fullscreen-popup.c,v 1.7 2005/10/20 13:12:56 chpe Exp $
  */
 
 #include "config.h"
 
 #include "ephy-fullscreen-popup.h"
 #include "ephy-spinner.h"
-#include "ephy-stock-icons.h"
 #include "ephy-debug.h"
 
 #include <glib/gi18n.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
-#include <gtk/gtkhseparator.h>
 #include <gtk/gtkeventbox.h>
 #include <gtk/gtktooltips.h>
 #include <gtk/gtkenums.h>
@@ -46,7 +44,6 @@ struct _EphyFullscreenPopupPrivate
 {
 	EphyWindow *window;
 	GtkTooltips *tooltips;
-	GtkWidget *sep;
 	GtkWidget *frame;
 	EphySpinner *spinner;
 	GtkWidget *lock;
@@ -100,14 +97,12 @@ static void
 ephy_fullscreen_popup_update_visibility (EphyFullscreenPopup *popup)
 {
 	EphyFullscreenPopupPrivate *priv = popup->priv;
-	gboolean show_frame, show_sep;
+	gboolean show_frame;
 
 	show_frame = priv->spinning || priv->show_lock;
-	show_sep = show_frame && priv->show_button;
 
 	g_object_set (priv->button, "visible", priv->show_button,
 				    "sensitive", priv->show_button, NULL);
-	g_object_set (priv->sep, "visible", show_sep, NULL);
 	g_object_set (priv->frame, "visible", show_frame, NULL);
 	g_object_set (priv->spinner, "visible", priv->spinning, NULL);
 	g_object_set (priv->lock_ebox, "visible", priv->show_lock, NULL);
@@ -232,7 +227,7 @@ ephy_fullscreen_popup_constructor (GType type,
 	EphyFullscreenPopup *popup;
 	EphyFullscreenPopupPrivate *priv;
 	GtkWindow *window;
-	GtkWidget *hbox, *frame_hbox, *button_hbox, *icon, *label;
+	GtkWidget *hbox, *frame_hbox, *icon;
 
 	object = parent_class->constructor (type, n_construct_properties,
 					    construct_params);
@@ -247,7 +242,7 @@ ephy_fullscreen_popup_constructor (GType type,
 	g_object_ref (G_OBJECT (priv->tooltips));
 	gtk_object_sink (GTK_OBJECT (priv->tooltips));
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_add (GTK_CONTAINER (window), hbox);
 	gtk_widget_show (hbox);
 
@@ -278,30 +273,16 @@ ephy_fullscreen_popup_constructor (GType type,
 	gtk_box_pack_start (GTK_BOX (frame_hbox), priv->lock_ebox, FALSE, FALSE, 0);
 	gtk_widget_show (priv->lock_ebox);
 
-	/* separator */
-	priv->sep = gtk_hseparator_new ();
-	gtk_box_pack_start (GTK_BOX (hbox), priv->sep, FALSE, FALSE, 0);
-	gtk_widget_show (priv->sep);
-
 	/* exit button */
-	priv->button = gtk_button_new ();
+	priv->button = gtk_button_new_with_label (_("Leave Fullscreen"));
+	icon = gtk_image_new_from_stock (GTK_STOCK_LEAVE_FULLSCREEN, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image (GTK_BUTTON (priv->button), icon);
+	/* don't show the image! see bug #307818 */
 	g_signal_connect (priv->button, "clicked",
 			  G_CALLBACK (exit_button_clicked_cb), popup);
 	gtk_box_pack_start (GTK_BOX (hbox), priv->button, FALSE, FALSE, 0);
 	gtk_widget_show (priv->button);
 
-	button_hbox = gtk_hbox_new (FALSE, 2);
-	gtk_container_add (GTK_CONTAINER (priv->button), button_hbox);
-	gtk_widget_show (button_hbox);
-
-	icon = gtk_image_new_from_stock (STOCK_LEAVE_FULLSCREEN, GTK_ICON_SIZE_BUTTON);
-	gtk_box_pack_start (GTK_BOX (button_hbox), icon, FALSE, FALSE, 0);
-	gtk_widget_show (icon);
-
-	label = gtk_label_new (_("Leave Fullscreen"));
-	gtk_box_pack_start (GTK_BOX (button_hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-
 	ephy_fullscreen_popup_update_visibility (popup);
 
 	return object;
