@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-location-action.c,v 1.36 2005/02/06 14:37:12 chpe Exp $
+ *  $Id: ephy-location-action.c,v 1.41 2005/10/06 18:28:55 chpe Exp $
  */
 
 #include "config.h"
@@ -24,15 +24,16 @@
 #include "ephy-location-action.h"
 #include "ephy-location-entry.h"
 #include "ephy-shell.h"
-#include "ephy-favicon-cache.h"
 #include "ephy-completion-model.h"
 #include "ephy-link.h"
 #include "ephy-debug.h"
 
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkentrycompletion.h>
+#include <gtk/gtkmain.h>
 
 #define EPHY_LOCATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_LOCATION_ACTION, EphyLocationActionPrivate))
 
@@ -41,14 +42,15 @@ struct _EphyLocationActionPrivate
 	EphyWindow *window;
 	GList *actions;
 	char *address;
-	gboolean editable;
+	char *typed_address;
 	EphyNode *smart_bmks;
 	EphyBookmarks *bookmarks;
-	char *icon;
-	EphyFaviconCache *cache;
+	GdkPixbuf *icon;
 	char *lock_stock_id;
 	char *lock_tooltip;
-	gboolean show_lock;
+	guint editable : 1;
+	guint show_lock : 1;
+	guint secure : 1;
 };
 
 static void ephy_location_action_init       (EphyLocationAction *action);
@@ -68,6 +70,7 @@ enum
 	PROP_ICON,
 	PROP_LOCK_STOCK,
 	PROP_LOCK_TOOLTIP,
+	PROP_SECURE,
 	PROP_SHOW_LOCK,
 	PROP_WINDOW
 };
@@ -130,7 +133,7 @@ action_activated_cb (GtkEntryCompletion *completion,
 	                (node, EPHY_NODE_BMK_PROP_LOCATION);
 		g_return_if_fail (smart_url != NULL);
 
-		url = ephy_bookmarks_solve_smart_url
+		url = ephy_bookmarks_resolve_address
 			(action->priv->bookmarks, smart_url, content);
 		g_return_if_fail (url != NULL);
 
@@ -142,16 +145,40 @@ action_activated_cb (GtkEntryCompletion *completion,
 }
 
 static void
-location_url_activate_cb (GtkEntry *entry,
-			  EphyLocationAction *action)
+entry_activate_cb (GtkEntry *entry,
+		   EphyLocationAction *action)
 {
+	EphyBookmarks *bookmarks;
+	GdkEvent *event;
+	gboolean control = FALSE;
 	const char *content;
+	char *address;
 
 	content = gtk_entry_get_text (entry);
-	if (content != NULL)
+	if (content == NULL || content[0] == '\0') return;
+
+	bookmarks = ephy_shell_get_bookmarks (ephy_shell_get_default ());
+
+	address = ephy_bookmarks_resolve_address (bookmarks, content, NULL);
+	g_return_if_fail (address != NULL);
+
+	event = gtk_get_current_event ();
+	if (event)
 	{
-		ephy_link_open (EPHY_LINK (action), content, NULL, 0);
+		if (event->type == GDK_KEY_PRESS ||
+		    event->type == GDK_KEY_RELEASE)
+		{
+			control = (event->key.state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK;
+		}
+			
+		gdk_event_free (event);
 	}
+
+	/* FIXME use ephy_bookmarks_resolve_address here too? */
+	ephy_link_open (EPHY_LINK (action), address, NULL, 
+			control ? EPHY_LINK_NEW_TAB : 0);
+
+	g_free (address);
 }
 
 static void
@@ -164,7 +191,7 @@ user_changed_cb (GtkWidget *proxy, EphyLocationAction *action)
 	LOG ("user_changed_cb, new address %s", address);
 
 	g_signal_handlers_block_by_func (action, G_CALLBACK (sync_address), proxy);
-	ephy_location_action_set_address (action, address);
+	ephy_location_action_set_address (action, address, NULL);
 	g_signal_handlers_unblock_by_func (action, G_CALLBACK (sync_address), proxy);
 }
 
@@ -181,12 +208,14 @@ sync_address (GtkAction *gaction,
 	      GtkWidget *proxy)
 {
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
 	EphyLocationEntry *lentry = EPHY_LOCATION_ENTRY (proxy);
 
 	LOG ("sync_address %s", action->priv->address);
 
 	g_signal_handlers_block_by_func (proxy, G_CALLBACK (user_changed_cb), action);
-	ephy_location_entry_set_location (lentry, action->priv->address);
+	ephy_location_entry_set_location (lentry, priv->address,
+					  priv->typed_address);
 	g_signal_handlers_unblock_by_func (proxy, G_CALLBACK (user_changed_cb), action);
 }
 
@@ -211,16 +240,8 @@ sync_icon (GtkAction *gaction,
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
 	EphyLocationActionPrivate *priv = action->priv;
 	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
-	GdkPixbuf *pixbuf;
 
-	pixbuf = ephy_favicon_cache_get (priv->cache, priv->icon);
-
-	ephy_location_entry_set_favicon (entry, pixbuf);
-
-	if (pixbuf != NULL)
-	{
-		g_object_unref (pixbuf);
-	}
+	ephy_location_entry_set_favicon (entry, priv->icon);
 }
 
 static void
@@ -247,6 +268,18 @@ sync_lock_tooltip (GtkAction *gaction,
 	ephy_location_entry_set_lock_tooltip (entry, priv->lock_tooltip);
 }
 
+static void
+sync_secure (GtkAction *gaction,
+	     GParamSpec *pspec,
+	     GtkWidget *proxy)
+{
+	EphyLocationAction *action = EPHY_LOCATION_ACTION (gaction);
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (proxy);
+
+	ephy_location_entry_set_secure (entry, priv->secure);
+}
+
 static void
 sync_show_lock (GtkAction *gaction,
 	        GParamSpec *pspec,
@@ -263,11 +296,13 @@ static char *
 get_location_cb (EphyLocationEntry *entry,
 		EphyLocationAction *action)
 {
-	EphyEmbed *embed;
+	EphyLocationActionPrivate *priv = action->priv;
+	EphyTab *tab;
 	
-	embed = ephy_window_get_active_embed (action->priv->window);
-	
-	return ephy_embed_get_location (embed, TRUE);
+	tab = ephy_window_get_active_tab (priv->window);
+	g_return_val_if_fail (tab != NULL, NULL);
+
+	return g_strdup (ephy_tab_get_address (tab));
 }
 
 static char *
@@ -366,15 +401,18 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 		sync_lock_tooltip (action, NULL, proxy);
 		g_signal_connect_object (action, "notify::lock-tooltip",
 					 G_CALLBACK (sync_lock_tooltip), proxy, 0);
+		sync_secure (action, NULL, proxy);
+		g_signal_connect_object (action, "notify::secure",
+					 G_CALLBACK (sync_secure), proxy, 0);
 		sync_show_lock (action, NULL, proxy);
 		g_signal_connect_object (action, "notify::show-lock",
 					 G_CALLBACK (sync_show_lock), proxy, 0);
 
 		entry = ephy_location_entry_get_entry (lentry);
 		g_signal_connect_object (entry, "activate",
-					 G_CALLBACK (location_url_activate_cb),
+					 G_CALLBACK (entry_activate_cb),
 					 action, 0);
-		g_signal_connect_object (proxy, "user_changed",
+		g_signal_connect_object (proxy, "user-changed",
 					 G_CALLBACK (user_changed_cb), action, 0);
 		g_signal_connect_object (proxy, "lock-clicked",
 					 G_CALLBACK (lock_clicked_cb), action, 0);
@@ -415,32 +453,39 @@ ephy_location_action_set_property (GObject *object,
 				   GParamSpec *pspec)
 {
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (object);
+	EphyLocationActionPrivate *priv = action->priv;
 
 	switch (prop_id)
 	{
 		case PROP_ADDRESS:
-			ephy_location_action_set_address (action, g_value_get_string (value));
+			ephy_location_action_set_address (action, g_value_get_string (value), NULL);
 			break;
 		case PROP_EDITABLE:
-			action->priv->editable = g_value_get_boolean (value);
+			priv->editable = g_value_get_boolean (value);
 			break;
 		case PROP_ICON:
-			g_free (action->priv->icon);
-			action->priv->icon = g_value_dup_string (value);
+			if (priv->icon != NULL)
+			{
+				g_object_unref (priv->icon);
+			}
+			priv->icon = GDK_PIXBUF (g_value_dup_object (value));
 			break;
 		case PROP_LOCK_STOCK:
-			g_free (action->priv->lock_stock_id);
-			action->priv->lock_stock_id = g_value_dup_string (value);
+			g_free (priv->lock_stock_id);
+			priv->lock_stock_id = g_value_dup_string (value);
 			break;
 		case PROP_LOCK_TOOLTIP:
-			g_free (action->priv->lock_tooltip);
-			action->priv->lock_tooltip = g_value_dup_string (value);
+			g_free (priv->lock_tooltip);
+			priv->lock_tooltip = g_value_dup_string (value);
+			break;
+		case PROP_SECURE:
+			priv->secure = g_value_get_boolean (value);
 			break;
 		case PROP_SHOW_LOCK:
-			action->priv->show_lock = g_value_get_boolean (value);
+			priv->show_lock = g_value_get_boolean (value);
 			break;
 		case PROP_WINDOW:
-			action->priv->window = EPHY_WINDOW (g_value_get_object (value));
+			priv->window = EPHY_WINDOW (g_value_get_object (value));
 			break;
 	}
 }
@@ -452,6 +497,7 @@ ephy_location_action_get_property (GObject *object,
 				   GParamSpec *pspec)
 {
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (object);
+	EphyLocationActionPrivate *priv = action->priv;
 
 	switch (prop_id)
 	{
@@ -459,19 +505,22 @@ ephy_location_action_get_property (GObject *object,
 			g_value_set_string (value, ephy_location_action_get_address (action));
 			break;
 		case PROP_EDITABLE:
-			g_value_set_boolean (value, action->priv->editable);
+			g_value_set_boolean (value, priv->editable);
 			break;
 		case PROP_ICON:
-			g_value_set_string (value, action->priv->icon);
+			g_value_set_object (value, priv->icon);
 			break;
 		case PROP_LOCK_STOCK:
-			g_value_set_string (value, action->priv->lock_stock_id);
+			g_value_set_string (value, priv->lock_stock_id);
 			break;
 		case PROP_LOCK_TOOLTIP:
-			g_value_set_string (value, action->priv->lock_tooltip);
+			g_value_set_string (value, priv->lock_tooltip);
+			break;
+		case PROP_SECURE:
+			g_value_set_boolean (value, priv->secure);
 			break;
 		case PROP_SHOW_LOCK:
-			g_value_set_boolean (value, action->priv->show_lock);
+			g_value_set_boolean (value, priv->show_lock);
 			break;
 		case PROP_WINDOW:
 			/* not readable */
@@ -522,11 +571,11 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
-					 g_param_spec_string  ("icon",
-							       "Icon",
-							       "The icon",
-							       NULL,
-							       G_PARAM_READWRITE));
+					 g_param_spec_object ("icon",
+							      "Icon",
+							      "The icon",
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_LOCK_STOCK,
@@ -544,6 +593,14 @@ ephy_location_action_class_init (EphyLocationActionClass *class)
 							       NULL,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_SECURE,
+					 g_param_spec_boolean ("secure",
+							       "Secure",
+							       "Secure",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_SHOW_LOCK,
 					 g_param_spec_boolean ("show-lock",
@@ -671,32 +728,30 @@ actions_child_changed_cb (EphyNode *node,
 static void
 ephy_location_action_init (EphyLocationAction *action)
 {
-	action->priv = EPHY_LOCATION_ACTION_GET_PRIVATE (action);
+	EphyLocationActionPrivate *priv;
 
-	action->priv->address = g_strdup ("");
-	action->priv->editable = TRUE;
-	action->priv->bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	action->priv->smart_bmks = ephy_bookmarks_get_smart_bookmarks
+	priv = action->priv = EPHY_LOCATION_ACTION_GET_PRIVATE (action);
+
+	priv->address = g_strdup ("");
+	priv->editable = TRUE;
+	priv->bookmarks = ephy_shell_get_bookmarks (ephy_shell);
+	priv->smart_bmks = ephy_bookmarks_get_smart_bookmarks
 		(action->priv->bookmarks);
 
 	init_actions_list (action);
 
-	ephy_node_signal_connect_object (action->priv->smart_bmks,
+	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_ADDED,
 			                 (EphyNodeCallback)actions_child_added_cb,
 			                 G_OBJECT (action));
-	ephy_node_signal_connect_object (action->priv->smart_bmks,
+	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_REMOVED,
 			                 (EphyNodeCallback)actions_child_removed_cb,
 			                 G_OBJECT (action));
-	ephy_node_signal_connect_object (action->priv->smart_bmks,
+	ephy_node_signal_connect_object (priv->smart_bmks,
 			                 EPHY_NODE_CHILD_CHANGED,
 			                 (EphyNodeCallback)actions_child_changed_cb,
 			                 G_OBJECT (action));
-
-	action->priv->cache = EPHY_FAVICON_CACHE
-		(ephy_embed_shell_get_favicon_cache (embed_shell));
-	g_object_ref (action->priv->cache);
 }
 
 static void
@@ -705,12 +760,16 @@ ephy_location_action_finalize (GObject *object)
 	EphyLocationAction *action = EPHY_LOCATION_ACTION (object);
 	EphyLocationActionPrivate *priv = action->priv;
 
+	if (priv->icon != NULL)
+	{
+		g_object_unref (priv->icon);
+	}
+
 	g_list_free (priv->actions);
 	g_free (priv->address);
-	g_free (priv->icon);
+	g_free (priv->typed_address);
 	g_free (priv->lock_stock_id);
 	g_free (priv->lock_tooltip);
-	g_object_unref (priv->cache);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -725,13 +784,22 @@ ephy_location_action_get_address (EphyLocationAction *action)
 
 void
 ephy_location_action_set_address (EphyLocationAction *action,
-				  const char *address)
+				  const char *address,
+				  const char *typed_address)
 {
+	EphyLocationActionPrivate *priv;
+
 	g_return_if_fail (EPHY_IS_LOCATION_ACTION (action));
 
+	priv = action->priv;
+
 	LOG ("set_address %s", address);
 
-	g_free (action->priv->address);
-	action->priv->address = g_strdup (address);
+	g_free (priv->address);
+	priv->address = g_strdup (address);
+
+	g_free (priv->typed_address);
+	priv->typed_address = typed_address ? g_strdup (typed_address) : NULL;
+
 	g_object_notify (G_OBJECT (action), "address");
 }
