@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-navigation-action.c,v 1.24 2005/01/28 22:24:46 chpe Exp $
+ *  $Id: ephy-navigation-action.c,v 1.28 2005/10/16 18:59:45 philipl Exp $
  */
 
 #include "config.h"
@@ -39,6 +39,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenutoolbutton.h>
 #include <gtk/gtktoolbar.h>
+#include <gtk/gtkstatusbar.h>
 
 #define NTH_DATA_KEY	"GoNTh"
 #define URL_DATA_KEY	"GoURL"
@@ -50,6 +51,7 @@ struct _EphyNavigationActionPrivate
 	EphyWindow *window;
 	EphyNavigationDirection direction;
 	char *arrow_tooltip;
+	guint statusbar_cid;
 };
 
 enum
@@ -136,18 +138,71 @@ new_history_menu_item (const char *origtext,
 }
 
 static void
-activate_back_or_forward_menu_item_cb (GtkWidget *menuitem,
-				       EphyNavigationAction *action)
+activate_by_history_index (EphyNavigationAction *action,
+			   int index)
 {
 	EphyEmbed *embed;
-	int go_nth;
 
 	embed = ephy_window_get_active_embed (action->priv->window);
 	g_return_if_fail (embed != NULL);
 
+	if (ephy_gui_is_middle_click ())
+	{
+		EphyEmbed *dest;
+		EphyTab *newTab;
+
+		newTab = ephy_link_open (EPHY_LINK (action), "about:blank", NULL,
+					 EPHY_LINK_NEW_TAB);
+		g_return_if_fail (newTab != NULL);
+
+		dest = ephy_tab_get_embed (newTab);
+		g_return_if_fail (dest != NULL);
+
+		ephy_embed_shistory_copy (embed, dest, TRUE, TRUE, FALSE);
+		embed = dest;
+	}
+	ephy_embed_shistory_go_nth (embed, index);
+}
+
+static void
+activate_back_or_forward_menu_item_cb (GtkWidget *menuitem,
+				       EphyNavigationAction *action)
+{
+	int go_nth;
+
 	go_nth = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), NTH_DATA_KEY));
 
-	ephy_embed_shistory_go_nth (embed, go_nth);
+	activate_by_history_index(action, go_nth);
+}
+
+static void
+select_menu_item_cb (GtkWidget *menuitem,
+	             EphyNavigationAction *action)
+{
+	char *url;
+	GtkWidget *statusbar;
+
+	url = g_object_get_data (G_OBJECT (menuitem), URL_DATA_KEY);
+	g_return_if_fail (url != NULL);
+
+	statusbar = ephy_window_get_statusbar (action->priv->window);
+
+	gtk_statusbar_push (GTK_STATUSBAR (statusbar), action->priv->statusbar_cid, url);
+}
+
+static void
+deselect_menu_item_cb (GtkWidget *menuitem,
+	               EphyNavigationAction *action)
+{
+	char *url;
+	GtkWidget *statusbar;
+
+	url = g_object_get_data (G_OBJECT (menuitem), URL_DATA_KEY);
+	g_return_if_fail (url != NULL);
+
+	statusbar = ephy_window_get_statusbar (action->priv->window);
+
+	gtk_statusbar_pop (GTK_STATUSBAR (statusbar), action->priv->statusbar_cid);
 }
 
 static void
@@ -210,14 +265,22 @@ build_back_or_forward_menu (EphyNavigationAction *action)
 
 		g_object_set_data (G_OBJECT (item), NTH_DATA_KEY,
 				   GINT_TO_POINTER (start));
+		g_object_set_data_full (G_OBJECT (item), URL_DATA_KEY, url, 
+					(GDestroyNotify) g_free);
+
 		g_signal_connect (item, "activate",
 				  G_CALLBACK (activate_back_or_forward_menu_item_cb),
 				  action);
+		g_signal_connect (item, "select",
+				  G_CALLBACK (select_menu_item_cb),
+				  action);
+		g_signal_connect (item, "deselect",
+				  G_CALLBACK (deselect_menu_item_cb),
+				  action);
 
 		gtk_menu_shell_append (menu, item);
 		gtk_widget_show_all (item);
 
-		g_free (url);
 		g_free (title);
 
 		if (start < end)
@@ -262,6 +325,10 @@ build_up_menu (EphyNavigationAction *action)
 					(GDestroyNotify) g_free);
 		g_signal_connect (item, "activate",
 				  G_CALLBACK (activate_up_menu_item_cb), action);
+		g_signal_connect (item, "select",
+				  G_CALLBACK (select_menu_item_cb), action);
+		g_signal_connect (item, "deselect",
+				  G_CALLBACK (deselect_menu_item_cb), action);
 
 		gtk_menu_shell_append (menu, item);
 		gtk_widget_show (item);
@@ -336,6 +403,36 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
 	GTK_ACTION_CLASS (parent_class)->connect_proxy (action, proxy);
 }
 
+static void
+ephy_navigation_action_activate (GtkAction *gtk_action)
+{
+	EphyNavigationAction *action = EPHY_NAVIGATION_ACTION (gtk_action);
+	EphyWindow *window = action->priv->window;
+	EphyEmbed *embed;
+	int pos;
+
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
+
+	pos = ephy_embed_shistory_get_pos (embed);
+
+	if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_BACK)
+	{
+		activate_by_history_index (action, pos - 1);
+	}
+	else if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_FORWARD)
+	{
+		activate_by_history_index (action, pos + 1);
+	}
+	else if (action->priv->direction == EPHY_NAVIGATION_DIRECTION_UP)
+	{
+		ephy_link_open (EPHY_LINK (action),
+				ephy_embed_get_go_up_list (embed)->data,
+				NULL,
+				ephy_gui_is_middle_click () ? EPHY_LINK_NEW_TAB : 0);
+	}
+}
+
 static void
 ephy_navigation_action_init (EphyNavigationAction *action)
 {
@@ -370,7 +467,18 @@ ephy_navigation_action_set_property (GObject *object,
 			nav->priv->direction = g_value_get_int (value);
 			break;
 		case PROP_WINDOW:
-			nav->priv->window = EPHY_WINDOW (g_value_get_object (value));
+			{
+				GtkWidget *statusbar;
+
+				nav->priv->window = EPHY_WINDOW (g_value_get_object (value));
+
+				/* statusbar context to display current selected item */
+				statusbar = ephy_window_get_statusbar (nav->priv->window);
+
+				nav->priv->statusbar_cid = gtk_statusbar_get_context_id (
+								GTK_STATUSBAR (statusbar), 
+								"navigation_message");
+			}
 			break;
 	}
 }
@@ -411,6 +519,7 @@ ephy_navigation_action_class_init (EphyNavigationActionClass *class)
 
 	action_class->toolbar_item_type = GTK_TYPE_MENU_TOOL_BUTTON;
 	action_class->connect_proxy = connect_proxy;
+	action_class->activate = ephy_navigation_action_activate;
 
 	g_object_class_install_property (object_class,
 					 PROP_ARROW_TOOLTIP,
