@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.130.4.2 2005/10/02 22:02:33 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.134 2005/10/16 20:29:16 chpe Exp $
  */
 
 #include "config.h"
@@ -30,6 +30,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-window.h"
+#include "ephy-bookmarks-ui.h"
 #include "ephy-bookmarks-import.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-history-window.h"
@@ -47,10 +48,7 @@
 #include "print-dialog.h"
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
-
-#ifdef ENABLE_DBUS
 #include "ephy-dbus.h"
-#endif
 
 #include <string.h>
 #include <bonobo/bonobo-main.h>
@@ -551,14 +549,12 @@ ephy_shell_dispose (GObject *object)
 		priv->extensions_manager = NULL;
 	}
 
-#ifdef ENABLE_DBUS
 	if (priv->dbus_service != NULL)
 	{
 		LOG ("Shutting down DBUS service");
 		g_object_unref (priv->dbus_service);
 		priv->dbus_service = NULL;
 	}
-#endif
 
 	if (priv->session != NULL)
 	{
@@ -651,7 +647,7 @@ ephy_shell_finalize (GObject *object)
 EphyShell *
 ephy_shell_get_default (void)
 {
-   return ephy_shell;
+	return ephy_shell;
 }
 
 EphyShell *
@@ -943,19 +939,10 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 	{
 		if (shell->priv->toolbars_model == NULL)
 		{
-			EphyBookmarks *bookmarks;
-			GObject *bookmarksbar_model;
-
 			shell->priv->toolbars_model = ephy_toolbars_model_new ();
-
-			/* get the bookmarks toolbars model. we have to do this
-			 * before loading the toolbars model from disk, since
-			 * this will connect the get_item_* signals
-			 */
-			bookmarks = ephy_shell_get_bookmarks (shell);
-			bookmarksbar_model = ephy_bookmarks_get_toolbars_model (bookmarks);
-
-			/* ok, now we can load the model */
+			
+			ephy_bookmarks_ui_attach_toolbar_model (shell->priv->toolbars_model);
+			
 			ephy_toolbars_model_load
 				(EPHY_TOOLBARS_MODEL (shell->priv->toolbars_model));
 		}
@@ -1091,7 +1078,6 @@ ephy_shell_get_print_setup_dialog (EphyShell *shell)
 	return shell->priv->print_setup_dialog;
 }
 
-#ifdef ENABLE_DBUS
 GObject	*
 ephy_shell_get_dbus_service (EphyShell *shell)
 {
@@ -1105,4 +1091,3 @@ ephy_shell_get_dbus_service (EphyShell *shell)
 
 	return G_OBJECT (shell->priv->dbus_service);
 }
-#endif
