@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.157 2005/08/20 22:22:00 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.163 2005/10/13 14:57:11 philipl Exp $
  */
 
 #include "config.h"
@@ -64,9 +64,15 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <string.h>
 
+#ifdef ENABLE_PYTHON
+#include "ephy-python.h"
+#endif
+
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
-#define MAX_HIDDEN_POPUPS 5
+#define MAX_HIDDEN_POPUPS	5
+#define RELOAD_DELAY		250 /* ms */
+#define MAX_TITLE_LENGTH	512 /* characters */
 
 struct _EphyTabPrivate
 {
@@ -834,6 +840,10 @@ ephy_tab_finalize (GObject *object)
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 
+#ifdef ENABLE_PYTHON
+	ephy_python_schedule_gc ();
+#endif
+
 	LOG ("EphyTab finalized %p", tab);
 }
 
@@ -1304,7 +1314,7 @@ ephy_tab_file_monitor_cb (GnomeVFSMonitorHandle *handle,
 			}
 
 			priv->reload_scheduled_id =
-				g_timeout_add (100 /* ms */,
+				g_timeout_add (RELOAD_DELAY,
 					       (GSourceFunc) ephy_file_monitor_reload_cb,
 					       tab);
 			break;
@@ -1324,11 +1334,12 @@ ephy_tab_update_file_monitor (EphyTab *tab,
 {
 	EphyTabPrivate *priv = tab->priv;
 	GnomeVFSMonitorHandle *handle = NULL;
-	GnomeVFSURI *uri;
 	gboolean local;
 
-	if (priv->address != NULL && address != NULL &&
+	if (priv->monitor != NULL &&
+	    priv->address != NULL && address != NULL &&
 	    strcmp (priv->address, address) == 0)
+	
 	{
 		/* same address, no change needed */
 		return;
@@ -1336,12 +1347,7 @@ ephy_tab_update_file_monitor (EphyTab *tab,
 
 	ephy_tab_file_monitor_cancel (tab);
 
-	uri = gnome_vfs_uri_new (address);
-	if (uri == NULL) return;
-
-	local = gnome_vfs_uri_is_local (uri);
-	gnome_vfs_uri_unref (uri);
-
+	local = g_str_has_prefix (address, "file://");
 	if (local == FALSE) return;
 	
 	if (gnome_vfs_monitor_add (&handle, address,
@@ -1799,26 +1805,6 @@ ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 	g_object_notify (G_OBJECT (tab), "visibility");
 }
 
-static void
-ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
-{
-	EphyWindow *window;
-	GtkWidget *notebook;
-
-	g_return_if_fail (EPHY_IS_TAB (tab));
-
-	LOG ("ephy_tab_destroy_browser_cb tab %p parent %p",
-	     tab, ((GtkWidget *) tab)->parent);
-
-	window = ephy_tab_get_window (tab);
-	g_return_if_fail (window != NULL);
-
-	/* Do not use ephy_window_remove_tab because it will
-	   check for unsubmitted forms */
-	notebook = ephy_window_get_notebook (window);
-	ephy_notebook_remove_tab (EPHY_NOTEBOOK (notebook), tab);
-}
-
 static gboolean
 open_link_in_new_tab (EphyTab *tab,
 		      const char *link_address)
@@ -1833,8 +1819,19 @@ open_link_in_new_tab (EphyTab *tab,
 
 	if (new_tab)
 	{
-		return ephy_link_open (EPHY_LINK (tab), link_address,
-				       tab, EPHY_LINK_NEW_TAB) != NULL;
+		EphyTab *dest;
+		dest = ephy_link_open (EPHY_LINK (tab), link_address,
+				       tab, EPHY_LINK_NEW_TAB);
+
+		if (dest)
+		{
+			ephy_embed_shistory_copy (ephy_tab_get_embed (tab),
+						  ephy_tab_get_embed (dest),
+						  TRUE,   /* back history */
+						  FALSE,  /* forward history */
+					  	  FALSE); /* current index */
+			return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -2049,9 +2046,6 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect_object (embed, "visibility",
 				 G_CALLBACK (ephy_tab_visibility_cb),
 				 tab, 0);
-	g_signal_connect_object (embed, "destroy_browser",
-				 G_CALLBACK (ephy_tab_destroy_brsr_cb),
-				 tab, 0);
 	g_signal_connect_object (embed, "ge_dom_mouse_click",
 				 G_CALLBACK (ephy_tab_dom_mouse_click_cb),
 				 tab, 0);
@@ -2194,7 +2188,7 @@ ephy_tab_set_title (EphyTab *tab,
 {
 	EphyTabPrivate *priv = tab->priv;
 
-	if (!priv->is_blank && (title == NULL || title[0] == '\0'))
+	if (!priv->is_blank && (title == NULL || g_strstrip (title)[0] == '\0'))
 	{
 
 		g_free (title);
@@ -2208,13 +2202,56 @@ ephy_tab_set_title (EphyTab *tab,
 			priv->is_blank = TRUE;
 		}
 	}
+	else if (priv->is_blank && title != NULL)
+	{
+		g_free (title);
+		title = NULL;
+	}
 
 	g_free (priv->title);
-	priv->title = title;
+	priv->title = ephy_string_shorten (title, MAX_TITLE_LENGTH);
 
 	g_object_notify (G_OBJECT (tab), "title");
 }
 
+/**
+ * ephy_tab_get_title_composite:
+ * @tab: an #EphyTab
+ *
+ * Returns the title of the web page loaded in @tab.
+ * 
+ * This differs from #ephy_tab_get_title in that this function
+ * will return a special title while the page is still loading.
+ *
+ * Return value: @tab's web page's title. Will never be %NULL.
+ **/
+const char *
+ephy_tab_get_title_composite (EphyTab *tab)
+{
+	EphyTabPrivate *priv;
+	const char *title = "";
+
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
+
+	priv = tab->priv;
+
+	if (priv->is_blank)
+	{
+		title = _("Blank page");
+	}
+	else if (priv->is_loading &&
+		 priv->loading_title != NULL)
+	{
+		title = priv->loading_title;
+	}
+	else
+	{
+		title = priv->title;
+	}
+
+	return title != NULL ? title : "";
+}
+
 /**
  * ephy_tab_get_title:
  * @tab: an #EphyTab
@@ -2237,11 +2274,6 @@ ephy_tab_get_title (EphyTab *tab)
 	{
 		title = _("Blank page");
 	}
-	else if (priv->is_loading &&
-		 priv->loading_title != NULL)
-	{
-		title = priv->loading_title;
-	}
 	else
 	{
 		title = priv->title;
