@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-toolbar.c,v 1.17.4.1 2005/10/20 13:13:54 chpe Exp $
+ *  $Id: ephy-toolbar.c,v 1.26 2005/10/21 16:15:09 philipl Exp $
  */
 
 #include "config.h"
@@ -76,11 +76,12 @@ struct _EphyToolbarPrivate
 	GtkToolItem *sep_item;
 	GtkToolItem *exit_button;
 	gulong set_focus_handler;
-	gboolean updating_address;
-	gboolean show_lock;
-	gboolean lock_visible;
-	gboolean leave_fullscreen_visible;
-	gboolean spinning;
+
+	guint updating_address : 1;
+	guint show_lock : 1;
+	guint is_secure : 1;
+	guint leave_fullscreen_visible : 1;
+	guint spinning : 1;
 };
 
 static const GtkTargetEntry drag_targets [] =
@@ -132,10 +133,9 @@ ephy_toolbar_update_fixed_visibility (EphyToolbar *toolbar)
 static void
 ephy_toolbar_update_spinner (EphyToolbar *toolbar)
 {
-	GtkWidget *widget = GTK_WIDGET (toolbar);
 	EphyToolbarPrivate *priv = toolbar->priv;
 
-	if (priv->spinning && GTK_WIDGET_VISIBLE (widget))
+	if (priv->spinning)
 	{
 		ephy_spinner_start (EPHY_SPINNER (priv->spinner));
 	}
@@ -166,20 +166,6 @@ fixed_toolbar_reconfigured_cb (GtkToolItem *item,
 	ephy_spinner_set_size (spinner, size);
 }
 
-static void
-toolbar_added_cb (EggToolbarsModel *model,
-		  int position,
-		  EggEditableToolbar *toolbar)
-{
-	const char *t_name;
-
-	t_name = egg_toolbars_model_toolbar_nth (model, position);
-	g_return_if_fail (t_name != NULL);
-
-	egg_editable_toolbar_set_drag_dest
-		(toolbar, drag_targets, G_N_ELEMENTS (drag_targets), t_name);
-}
-
 static void 
 maybe_finish_activation_cb (EphyWindow *window,
 			    GtkWidget *widget,
@@ -262,9 +248,9 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 	action = priv->actions[BACK_ACTION] =
 		g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
 			      "name", "NavigationBack",
-			      "label", _("Back"),
+			      "label", _("_Back"),
 			      "stock_id", GTK_STOCK_GO_BACK,
-			      "tooltip", _("Go back"),
+			      "tooltip", _("Go to the previous visited page"),
 			      /* this is the tooltip on the Back button's drop-down arrow, which will show
 			       * a menu with all sites you can go 'back' to
 			       */
@@ -273,17 +259,18 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			      "direction", EPHY_NAVIGATION_DIRECTION_BACK,
 			      "is_important", TRUE,
 			      NULL);
-	g_signal_connect (action, "activate",
-			  G_CALLBACK (window_cmd_go_back), priv->window);
-	gtk_action_group_add_action (priv->action_group, action);
+	g_signal_connect_swapped (action, "open-link",
+				  G_CALLBACK (ephy_link_open), toolbar);
+	gtk_action_group_add_action_with_accel (priv->action_group, action,
+						"<alt>Left");
 	g_object_unref (action);
 
 	action = priv->actions[FORWARD_ACTION] =
 		g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
 			      "name", "NavigationForward",
-			      "label", _("Forward"),
+			      "label", _("_Forward"),
 			      "stock_id", GTK_STOCK_GO_FORWARD,
-			      "tooltip", _("Go forward"),
+			      "tooltip", _("Go to the next visited page"),
 			      /* this is the tooltip on the Forward button's drop-down arrow, which will show
 			       * a menu with all sites you can go 'forward' to
 			       */
@@ -291,15 +278,16 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			      "window", priv->window,
 			      "direction", EPHY_NAVIGATION_DIRECTION_FORWARD,
 			      NULL);
-	g_signal_connect (action, "activate",
-			  G_CALLBACK (window_cmd_go_forward), priv->window);
-	gtk_action_group_add_action (priv->action_group, action);
+	g_signal_connect_swapped (action, "open-link",
+				  G_CALLBACK (ephy_link_open), toolbar);
+	gtk_action_group_add_action_with_accel (priv->action_group, action,
+						"<alt>Right");
 	g_object_unref (action);
 
 	action = priv->actions[UP_ACTION] =
 		g_object_new (EPHY_TYPE_NAVIGATION_ACTION,
 			      "name", "NavigationUp",
-			      "label", _("Up"),
+			      "label", _("_Up"),
 			      "stock_id", GTK_STOCK_GO_UP,
 			      "tooltip", _("Go up one level"),
 			      /* this is the tooltip on the Up button's drop-down arrow, which will show
@@ -309,11 +297,10 @@ ephy_toolbar_set_window (EphyToolbar *toolbar,
 			      "window", priv->window,
 			      "direction", EPHY_NAVIGATION_DIRECTION_UP,
 			      NULL);
-	g_signal_connect (action, "activate",
-			  G_CALLBACK (window_cmd_go_up), priv->window);
 	g_signal_connect_swapped (action, "open-link",
 				  G_CALLBACK (ephy_link_open), toolbar);
-	gtk_action_group_add_action (priv->action_group, action);
+	gtk_action_group_add_action_with_accel (priv->action_group, action,
+						"<alt>Up");
 	g_object_unref (action);
 
 	/* FIXME: I'm still waiting for the exact term to 
@@ -391,7 +378,7 @@ ephy_toolbar_set_show_leave_fullscreen (EphyToolbar *toolbar,
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 
-	priv->leave_fullscreen_visible = show;
+	priv->leave_fullscreen_visible = show != FALSE;
 
 	ephy_toolbar_update_fixed_visibility (toolbar);
 }
@@ -471,41 +458,31 @@ ephy_toolbar_set_navigation_actions (EphyToolbar *toolbar,
 
 void
 ephy_toolbar_set_security_state (EphyToolbar *toolbar,
+				 gboolean is_secure,
 				 gboolean show_lock,
 				 const char *stock_id,
 				 const char *tooltip)
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 
-	priv->show_lock = show_lock;
+	priv->show_lock = show_lock != FALSE;
+	priv->is_secure = is_secure != FALSE;
 
 	g_object_set (priv->actions[LOCATION_ACTION],
 		      "lock-stock-id", stock_id,
 		      "lock-tooltip", tooltip,
-		      "show-lock", priv->lock_visible && priv->show_lock,
+		      "show-lock", priv->show_lock,
+		      "secure", is_secure,
 		      NULL);
 }
 
-void
-ephy_toolbar_set_lock_visibility (EphyToolbar *toolbar,
-				  gboolean visible)
-{
-	EphyToolbarPrivate *priv = toolbar->priv;
-
-	priv->lock_visible = visible;
-
-	g_object_set (priv->actions[LOCATION_ACTION],
-		      "show-lock", priv->lock_visible && priv->show_lock,
-		      NULL);
-}
-			    
 void
 ephy_toolbar_set_spinning (EphyToolbar *toolbar,
 			   gboolean spinning)
 {
 	EphyToolbarPrivate *priv = toolbar->priv;
 
-	priv->spinning = spinning;
+	priv->spinning = spinning != FALSE;
 
 	ephy_toolbar_update_spinner (toolbar);
 }
@@ -526,42 +503,12 @@ ephy_toolbar_set_zoom (EphyToolbar *toolbar,
 static void
 ephy_toolbar_realize (GtkWidget *widget)
 {
-	EggEditableToolbar *etoolbar = EGG_EDITABLE_TOOLBAR (widget);
-	EphyToolbar *toolbar = EPHY_TOOLBAR (widget);
-	EggToolbarsModel *model = egg_editable_toolbar_get_model (etoolbar);
-	int i, n_toolbars;
-
 	GTK_WIDGET_CLASS (parent_class)->realize (widget);
-
-	g_signal_connect_after (model, "toolbar_added",
-				G_CALLBACK (toolbar_added_cb), toolbar);
-
-	/* now that the toolbar has been constructed, set drag dests */
-	n_toolbars = egg_toolbars_model_n_toolbars (model);
-	for (i = 0; i < n_toolbars; i++)
-	{
-		const char *t_name;
-
-		t_name = egg_toolbars_model_toolbar_nth (model, i);
-		g_return_if_fail (t_name != NULL);
-
-		egg_editable_toolbar_set_drag_dest
-			(etoolbar, drag_targets, G_N_ELEMENTS (drag_targets), t_name);
-	}	
 }
 
 static void
 ephy_toolbar_unrealize (GtkWidget *widget)
 {
-	EggEditableToolbar *eggtoolbar = EGG_EDITABLE_TOOLBAR (widget);
-	EphyToolbar *toolbar = EPHY_TOOLBAR (widget);
-	EggToolbarsModel *model;
- 
- 	model = egg_editable_toolbar_get_model (eggtoolbar);
-
-	g_signal_handlers_disconnect_by_func
-		(model, G_CALLBACK (toolbar_added_cb), toolbar);
-
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
 
@@ -588,7 +535,9 @@ ephy_toolbar_hide (GtkWidget *widget)
 static void
 ephy_toolbar_init (EphyToolbar *toolbar)
 {
-	toolbar->priv = EPHY_TOOLBAR_GET_PRIVATE (toolbar);
+	EphyToolbarPrivate *priv;
+
+	priv = toolbar->priv = EPHY_TOOLBAR_GET_PRIVATE (toolbar);
 }
 
 static GObject *
@@ -643,7 +592,6 @@ ephy_toolbar_finalize (GObject *object)
 {
 	EphyToolbar *toolbar = EPHY_TOOLBAR (object);
 	EphyToolbarPrivate *priv = toolbar->priv;
-	EggEditableToolbar *etoolbar = EGG_EDITABLE_TOOLBAR (object);
 
 	if (priv->set_focus_handler != 0)
 	{
@@ -651,10 +599,6 @@ ephy_toolbar_finalize (GObject *object)
 					     priv->set_focus_handler);
 	}
 
-	g_signal_handlers_disconnect_by_func
-		(egg_editable_toolbar_get_model (etoolbar),
-		 G_CALLBACK (toolbar_added_cb), toolbar);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
