@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-toolbar.h,v 1.3 2005/01/26 15:06:42 chpe Exp $
+ *  $Id: ephy-toolbar.h,v 1.7 2005/10/17 21:21:40 chpe Exp $
  */
 
 #ifndef EPHY_TOOLBAR_H
@@ -25,6 +25,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "egg-editable-toolbar.h"
 #include "ephy-window.h"
@@ -65,7 +66,7 @@ GType		ephy_toolbar_get_type			(void);
 EphyToolbar    *ephy_toolbar_new			(EphyWindow *window);
 
 void		ephy_toolbar_set_favicon		(EphyToolbar *toolbar,
-							 const char *icon);
+							 GdkPixbuf *icon);
 
 void		ephy_toolbar_set_show_leave_fullscreen	(EphyToolbar *toolbar,
 							 gboolean show);
@@ -75,8 +76,8 @@ void		ephy_toolbar_activate_location		(EphyToolbar *t);
 const char     *ephy_toolbar_get_location		(EphyToolbar *t);
 
 void		ephy_toolbar_set_location		(EphyToolbar *toolbar,
-							 const char *location);
-
+							 const char *address,
+							 const char *typed_address);
 
 void		ephy_toolbar_set_navigation_actions	(EphyToolbar *toolbar,
 							 gboolean back,
@@ -84,13 +85,11 @@ void		ephy_toolbar_set_navigation_actions	(EphyToolbar *toolbar,
 							 gboolean up);
 
 void		ephy_toolbar_set_security_state		(EphyToolbar *toolbar,
+							 gboolean is_secure,
 							 gboolean show_lock,
 							 const char *stock_id,
 							 const char *tooltip);
 
-void		ephy_toolbar_set_lock_visibility	(EphyToolbar *toolbar,
-							 gboolean visible);
-
 void		ephy_toolbar_set_spinning		(EphyToolbar *toolbar,
 							 gboolean spinning);
 
