@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-window.c,v 1.304 2005/08/20 22:24:48 chpe Exp $
+ *  $Id: ephy-window.c,v 1.317 2005/10/21 16:15:10 philipl Exp $
  */
 
 #include "config.h"
@@ -26,8 +26,6 @@
 #include "ephy-type-builtins.h"
 #include "ephy-embed-type-builtins.h"
 #include "ephy-command-manager.h"
-#include "ephy-bookmarks-menu.h"
-#include "ephy-favorites-menu.h"
 #include "ephy-state.h"
 #include "ppview-toolbar.h"
 #include "window-commands.h"
@@ -43,12 +41,12 @@
 #include "ephy-statusbar.h"
 #include "egg-editable-toolbar.h"
 #include "ephy-toolbar.h"
-#include "ephy-bookmarksbar.h"
 #include "popup-commands.h"
 #include "ephy-encoding-menu.h"
 #include "ephy-tabs-menu.h"
 #include "ephy-stock-icons.h"
 #include "ephy-extension.h"
+#include "ephy-bookmarks-ui.h"
 #include "ephy-link.h"
 #include "ephy-gui.h"
 #include "ephy-notebook.h"
@@ -73,6 +71,10 @@
 #include <X11/XF86keysym.h>
 #endif
 
+#ifdef ENABLE_PYTHON
+#include "ephy-python.h"
+#endif
+
 static void ephy_window_class_init		(EphyWindowClass *klass);
 static void ephy_window_link_iface_init		(EphyLinkIface *iface);
 static void ephy_window_init			(EphyWindow *gs);
@@ -90,11 +92,9 @@ static void ephy_window_notebook_switch_page_cb	(GtkNotebook *notebook,
 						 guint page_num,
 						 EphyWindow *window);
 static void ephy_window_view_statusbar_cb       (GtkAction *action,
-			                         EphyWindow *window);
+						 EphyWindow *window);
 static void ephy_window_view_toolbar_cb         (GtkAction *action,
-			                         EphyWindow *window);
-static void ephy_window_view_bookmarksbar_cb    (GtkAction *action,
-			                         EphyWindow *window);
+						 EphyWindow *window);
 static void ephy_window_view_popup_windows_cb	(GtkAction *action,
 						 EphyWindow *window);
 static void sync_tab_load_status		(EphyTab *tab,
@@ -148,7 +148,7 @@ static const GtkActionEntry ephy_menu_entries [] = {
 	{ "FilePrint", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
 	  N_("Print the current page"),
 	  G_CALLBACK (window_cmd_file_print) },
-	{ "FileSendTo", STOCK_SEND_MAIL, N_("S_end To..."), "<control>M",
+	{ "FileSendTo", STOCK_SEND_MAIL, N_("S_end Link by Email..."), "<control>M",
 	  N_("Send a link of the current page"),
 	  G_CALLBACK (window_cmd_file_send_to) },
 	{ "FileCloseTab", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
@@ -234,15 +234,6 @@ static const GtkActionEntry ephy_menu_entries [] = {
 
 	/* Go menu */
 
-	{ "GoBack", GTK_STOCK_GO_BACK, N_("_Back"), "<alt>Left",
-	  N_("Go to the previous visited page"),
-	  G_CALLBACK (window_cmd_go_back) },
-	{ "GoForward", GTK_STOCK_GO_FORWARD, N_("_Forward"), "<alt>Right",
-	  N_("Go to the next visited page"),
-	  G_CALLBACK (window_cmd_go_forward) },
-	{ "GoUp", GTK_STOCK_GO_UP, N_("_Up"), "<alt>Up",
-	  N_("Go up one level"),
-	  G_CALLBACK (window_cmd_go_up) },
 	{ "GoLocation", NULL, N_("_Location..."), "<control>L",
 	  N_("Go to a specified location"),
 	  G_CALLBACK (window_cmd_go_location) },
@@ -291,13 +282,10 @@ static const GtkToggleActionEntry ephy_menu_toggle_entries [] =
 	{ "ViewToolbar", NULL, N_("_Toolbar"), "<shift><control>T",
 	  N_("Show or hide toolbar"),
 	  G_CALLBACK (ephy_window_view_toolbar_cb), TRUE },
-	{ "ViewBookmarksBar", NULL, N_("_Bookmarks Bar"), NULL,
-	  N_("Show or hide bookmarks bar"),
-	  G_CALLBACK (ephy_window_view_bookmarksbar_cb), TRUE },
 	{ "ViewStatusbar", NULL, N_("St_atusbar"), NULL,
 	  N_("Show or hide statusbar"),
 	  G_CALLBACK (ephy_window_view_statusbar_cb), TRUE },
-	{ "ViewFullscreen", STOCK_FULLSCREEN, N_("_Fullscreen"), "F11",
+	{ "ViewFullscreen", GTK_STOCK_FULLSCREEN, N_("_Fullscreen"), "F11",
 	  N_("Browse at full screen"),
 	  G_CALLBACK (window_cmd_view_fullscreen), FALSE },
 	{ "ViewPopupWindows", EPHY_STOCK_POPUPS, N_("Popup _Windows"), NULL,
@@ -311,8 +299,6 @@ static const GtkToggleActionEntry ephy_menu_toggle_entries [] =
 static const GtkActionEntry ephy_popups_entries [] = {
 	/* Document */
 
-	{ "SaveBackgroundAs", NULL, N_("_Save Background As..."), NULL,
-	  NULL, G_CALLBACK (popup_cmd_save_background_as) },
 	{ "ContextBookmarkPage", STOCK_ADD_BOOKMARK, N_("Add Boo_kmark..."), "<control>D",
 	  N_("Add a bookmark for the current page"),
 	  G_CALLBACK (window_cmd_file_bookmark_page) },
@@ -395,7 +381,7 @@ static const struct
 #define BOOKMARKS_MENU_PATH "/menubar/BookmarksMenu"
 
 /* Until https://bugzilla.mozilla.org/show_bug.cgi?id=296002 is fixed */
-#define KEEP_TAB_IN_SAME_TOPLEVEL
+//#define KEEP_TAB_IN_SAME_TOPLEVEL
 
 #define EPHY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_WINDOW, EphyWindowPrivate))
 
@@ -405,15 +391,12 @@ struct _EphyWindowPrivate
 	GtkWidget *menu_dock;
 	GtkWidget *fullscreen_popup;
 	EphyToolbar *toolbar;
-	GtkWidget *bookmarksbar;
 	GtkWidget *statusbar;
 	GtkUIManager *manager;
 	GtkActionGroup *action_group;
 	GtkActionGroup *popups_action_group;
-	EphyFavoritesMenu *fav_menu;
 	EphyEncodingMenu *enc_menu;
 	EphyTabsMenu *tabs_menu;
-	EphyBookmarksMenu *bmk_menu;
 	PPViewToolbar *ppview_toolbar;
 	GtkNotebook *notebook;
 	EphyTab *active_tab;
@@ -423,6 +406,7 @@ struct _EphyWindowPrivate
 	guint help_message_cid;
 	EphyEmbedChrome chrome;
 	guint idle_resize_handler;
+	GHashTable *tabs_to_remove;
 	EphyEmbedEvent *context_event;
 	guint idle_worker;
 
@@ -461,22 +445,22 @@ static GObjectClass *parent_class = NULL;
 GType
 ephy_window_get_type (void)
 {
-        static GType type = 0;
+	static GType type = 0;
 
-        if (G_UNLIKELY (type == 0))
-        {
-                static const GTypeInfo our_info =
-                {
-                        sizeof (EphyWindowClass),
-                        NULL, /* base_init */
-                        NULL, /* base_finalize */
-                        (GClassInitFunc) ephy_window_class_init,
-                        NULL,
-                        NULL, /* class_data */
-                        sizeof (EphyWindow),
-                        0, /* n_preallocs */
-                        (GInstanceInitFunc) ephy_window_init
-                };
+	if (G_UNLIKELY (type == 0))
+	{
+		static const GTypeInfo our_info =
+		{
+			sizeof (EphyWindowClass),
+			NULL, /* base_init */
+			NULL, /* base_finalize */
+			(GClassInitFunc) ephy_window_class_init,
+			NULL,
+			NULL, /* class_data */
+			sizeof (EphyWindow),
+			0, /* n_preallocs */
+			(GInstanceInitFunc) ephy_window_init
+		};
 		static const GInterfaceInfo link_info = 
 		{
 			(GInterfaceInitFunc) ephy_window_link_iface_init,
@@ -484,16 +468,16 @@ ephy_window_get_type (void)
 			NULL
 		};
 
-                type = g_type_register_static (GTK_TYPE_WINDOW,
+		type = g_type_register_static (GTK_TYPE_WINDOW,
 					       "EphyWindow",
 					       &our_info, 0);
 
 		g_type_add_interface_static (type,
 					     EPHY_TYPE_LINK,
 					     &link_info);
-        }
+	}
 
-        return type;
+	return type;
 }
 
 static void
@@ -538,7 +522,6 @@ get_chromes_visibility (EphyWindow *window,
 			gboolean *show_menubar,
 			gboolean *show_statusbar,
 			gboolean *show_toolbar,
-			gboolean *show_bookmarksbar,
 			gboolean *show_tabsbar)
 {
 	EphyWindowPrivate *priv = window->priv;
@@ -548,14 +531,13 @@ get_chromes_visibility (EphyWindow *window,
 	{
 		*show_menubar = *show_statusbar
 			      = *show_toolbar
-			      = *show_bookmarksbar
 			      = *show_tabsbar
 			      = FALSE;
 	}
 	else if (window->priv->fullscreen_mode)
 	{
 		*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
-		*show_menubar = *show_statusbar = *show_bookmarksbar = FALSE;
+		*show_menubar = *show_statusbar = FALSE;
 		*show_tabsbar = !priv->is_popup;
 	}
 	else
@@ -563,7 +545,6 @@ get_chromes_visibility (EphyWindow *window,
 		*show_menubar = (flags & EPHY_EMBED_CHROME_MENUBAR) != 0;
 		*show_statusbar = (flags & EPHY_EMBED_CHROME_STATUSBAR) != 0;
 		*show_toolbar = (flags & EPHY_EMBED_CHROME_TOOLBAR) != 0;
-		*show_bookmarksbar = (flags & EPHY_EMBED_CHROME_BOOKMARKSBAR) != 0;
 		*show_tabsbar = !priv->is_popup;
 	}
 }
@@ -573,24 +554,21 @@ sync_chromes_visibility (EphyWindow *window)
 {
 	EphyWindowPrivate *priv = window->priv;
 	GtkWidget *menubar;
-	gboolean show_statusbar, show_menubar, show_toolbar, show_bookmarksbar, show_tabsbar;
+	gboolean show_statusbar, show_menubar, show_toolbar, show_tabsbar;
 
 	if (priv->closing) return;
 
 	get_chromes_visibility (window, &show_menubar,
 				&show_statusbar, &show_toolbar,
-				&show_bookmarksbar, &show_tabsbar);
+				&show_tabsbar);
 
 	menubar = gtk_ui_manager_get_widget (window->priv->manager, "/menubar");
 	g_assert (menubar != NULL);
 
 	g_object_set (menubar, "visible", show_menubar, NULL);
 	g_object_set (priv->toolbar, "visible", show_toolbar, NULL);
-	g_object_set (priv->bookmarksbar, "visible", show_bookmarksbar, NULL);
 	g_object_set (priv->statusbar, "visible", show_statusbar, NULL);
 
-	ephy_toolbar_set_lock_visibility (priv->toolbar, !show_statusbar);
-
 	ephy_notebook_set_show_tabs (EPHY_NOTEBOOK (priv->notebook), show_tabsbar);
 
 	if (priv->fullscreen_popup != NULL)
@@ -650,11 +628,10 @@ ephy_window_unfullscreen (EphyWindow *window)
 	sync_chromes_visibility (window);
 }
 
-static gboolean
-confirm_close_with_modified_forms (EphyWindow *window)
+static GtkWidget *
+construct_confirm_close_dialog (EphyWindow *window)
 {
 	GtkWidget *dialog;
-	int response;
 
 	dialog = gtk_message_dialog_new
 		(GTK_WINDOW (window),
@@ -673,11 +650,21 @@ confirm_close_with_modified_forms (EphyWindow *window)
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
-	/* FIXME set title */
+	/* FIXME gtk_window_set_title (GTK_WINDOW (dialog), _("Close Document?")); */
 	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
 
 	gtk_window_group_add_window (GTK_WINDOW (window)->group, GTK_WINDOW (dialog));
 
+	return dialog;
+}
+
+static gboolean
+confirm_close_with_modified_forms (EphyWindow *window)
+{
+	GtkWidget *dialog;
+	int response;
+
+	dialog = construct_confirm_close_dialog (window);
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	gtk_widget_destroy (dialog);
@@ -755,7 +742,7 @@ ephy_window_key_press_event (GtkWidget *widget,
 	}
 
 	/* Show and activate the menubar, if it isn't visible */
-        if (event->keyval == keyval && (event->state & mask) == (modifier & mask))
+	if (event->keyval == keyval && (event->state & mask) == (modifier & mask))
 	{
 		menubar = gtk_ui_manager_get_widget (window->priv->manager, "/menubar");
 		g_return_val_if_fail (menubar != NULL , FALSE);
@@ -770,14 +757,14 @@ ephy_window_key_press_event (GtkWidget *widget,
 
 			return TRUE;
 		}
-        }
+	}
 
 	return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
 static gboolean
 ephy_window_delete_event (GtkWidget *widget,
-			  GdkEvent *event)
+			  GdkEventAny *event)
 {
 	EphyWindow *window = EPHY_WINDOW (widget);
 	EphyWindowPrivate *priv = window->priv;
@@ -842,6 +829,24 @@ ephy_window_delete_event (GtkWidget *widget,
 	return FALSE;
 }
 
+static void
+update_image_actions_visibility (EphyWindow *window, gboolean show)
+{
+	GtkAction *action;
+	GtkActionGroup *action_group;
+
+	action_group = window->priv->popups_action_group;
+
+	action = gtk_action_group_get_action (action_group, "OpenImage");
+	gtk_action_set_visible (action, show);
+	action = gtk_action_group_get_action (action_group, "SaveImageAs");
+	gtk_action_set_visible (action, show);
+	action = gtk_action_group_get_action (action_group, "SetImageAsBackground");
+	gtk_action_set_visible (action, show);
+	action = gtk_action_group_get_action (action_group, "CopyImageLocation");
+	gtk_action_set_visible (action, show);
+}
+
 static void
 update_edit_actions_sensitivity (EphyWindow *window, gboolean hide)
 {
@@ -982,6 +987,94 @@ menu_item_deselect_cb (GtkMenuItem *proxy,
 			   window->priv->help_message_cid);
 }
 
+static gboolean
+tool_item_enter_cb (GtkWidget *proxy,
+		    GdkEventCrossing *event,
+		    EphyWindow *window)
+{
+	if (event->mode == GDK_CROSSING_NORMAL && 
+	    event->detail == GDK_NOTIFY_NONLINEAR)
+	{
+		GtkToolItem *item;
+		GtkAction *action;
+		char *message;
+    
+		item = GTK_TOOL_ITEM (gtk_widget_get_ancestor (proxy, GTK_TYPE_TOOL_ITEM));
+		
+		action = g_object_get_data (G_OBJECT (item),  "gtk-action");
+		g_return_val_if_fail (action != NULL, FALSE);
+		
+		g_object_get (G_OBJECT (action), "tooltip", &message, NULL);
+		if (message)
+		{
+			gtk_statusbar_push (GTK_STATUSBAR (window->priv->statusbar),
+					    window->priv->help_message_cid, message);
+			g_free (message);
+		}
+	}
+	
+	return FALSE;
+}
+
+static gboolean
+tool_item_leave_cb (GtkWidget *proxy,
+		    GdkEventCrossing *event,
+		    EphyWindow *window)
+{
+	if (event->mode == GDK_CROSSING_NORMAL && 
+	    event->detail == GDK_NOTIFY_NONLINEAR)
+	{
+		gtk_statusbar_pop (GTK_STATUSBAR (window->priv->statusbar),
+				   window->priv->help_message_cid);
+	}
+	
+	return FALSE;
+}
+
+static void
+tool_item_drag_begin_cb (GtkWidget          *widget,
+			 GdkDragContext     *context,
+			 EphyWindow         *window)
+{
+	gtk_statusbar_pop (GTK_STATUSBAR (window->priv->statusbar),
+			   window->priv->help_message_cid);
+}
+
+
+static void
+connect_tool_item (GtkWidget *proxy, EphyWindow *window)
+{
+	if (GTK_IS_CONTAINER (proxy))
+	{
+		gtk_container_foreach (GTK_CONTAINER (proxy),
+				       (GtkCallback) connect_tool_item,
+				       (gpointer) window);
+	}
+
+	g_signal_connect (proxy, "drag_begin",
+			  G_CALLBACK (tool_item_drag_begin_cb), window);
+	 g_signal_connect (proxy, "enter-notify-event",
+			  G_CALLBACK (tool_item_enter_cb), window);
+	g_signal_connect (proxy, "leave-notify-event",
+			  G_CALLBACK (tool_item_leave_cb), window);
+}
+
+static void
+disconnect_tool_item (GtkWidget *proxy, EphyWindow *window)
+{
+	if (GTK_IS_CONTAINER (proxy))
+	{
+		gtk_container_foreach (GTK_CONTAINER (proxy),
+				       (GtkCallback) disconnect_tool_item,
+				       (gpointer) window);
+	}
+
+	g_signal_handlers_disconnect_by_func
+	  (proxy, G_CALLBACK (tool_item_enter_cb), window);
+	g_signal_handlers_disconnect_by_func
+	  (proxy, G_CALLBACK (tool_item_leave_cb), window);
+}
+
 static void
 disconnect_proxy_cb (GtkUIManager *manager,
 		     GtkAction *action,
@@ -995,6 +1088,10 @@ disconnect_proxy_cb (GtkUIManager *manager,
 		g_signal_handlers_disconnect_by_func
 			(proxy, G_CALLBACK (menu_item_deselect_cb), window);
 	}
+	else if (GTK_IS_TOOL_ITEM (proxy))
+	{
+		disconnect_tool_item (proxy, window);
+	}
 }
 
 static void
@@ -1010,6 +1107,10 @@ connect_proxy_cb (GtkUIManager *manager,
 		g_signal_connect (proxy, "deselect",
 				  G_CALLBACK (menu_item_deselect_cb), window);
 	}
+	else if (GTK_IS_TOOL_ITEM (proxy))
+	{
+		connect_tool_item (proxy, window);
+	}
 }
 
 static void
@@ -1017,11 +1118,11 @@ update_chromes_actions (EphyWindow *window)
 {
 	GtkActionGroup *action_group = window->priv->action_group;
 	GtkAction *action;
-	gboolean show_statusbar, show_menubar, show_toolbar, show_bookmarksbar, show_tabsbar;
+	gboolean show_statusbar, show_menubar, show_toolbar, show_tabsbar;
 
 	get_chromes_visibility (window, &show_menubar,
 				&show_statusbar, &show_toolbar,
-				&show_bookmarksbar, &show_tabsbar);
+				&show_tabsbar);
 
 	action = gtk_action_group_get_action (action_group, "ViewToolbar");
 	g_signal_handlers_block_by_func (G_OBJECT (action),
@@ -1032,15 +1133,6 @@ update_chromes_actions (EphyWindow *window)
 		 			   G_CALLBACK (ephy_window_view_toolbar_cb),
 		 			   window);
 
-	action = gtk_action_group_get_action (action_group, "ViewBookmarksBar");
-	g_signal_handlers_block_by_func (G_OBJECT (action),
-		 			 G_CALLBACK (ephy_window_view_bookmarksbar_cb),
-		 			 window);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show_bookmarksbar);
-	g_signal_handlers_unblock_by_func (G_OBJECT (action),
-		 			   G_CALLBACK (ephy_window_view_bookmarksbar_cb),
-		 			   window);
-
 	action = gtk_action_group_get_action (action_group, "ViewStatusbar");
 	g_signal_handlers_block_by_func (G_OBJECT (action),
 		 			 G_CALLBACK (ephy_window_view_statusbar_cb),
@@ -1115,6 +1207,8 @@ setup_ui_manager (EphyWindow *window)
 					     window);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	window->priv->action_group = action_group;
+	g_object_unref (action_group);
+
 	action = gtk_action_group_get_action (action_group, "FileOpen");
 	g_object_set (action, "short_label", _("Open"), NULL);
 	action = gtk_action_group_get_action (action_group, "FileSaveAs");
@@ -1142,6 +1236,7 @@ setup_ui_manager (EphyWindow *window)
 				      G_N_ELEMENTS (ephy_popups_entries), window);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	window->priv->popups_action_group = action_group;
+	g_object_unref (action_group);
 
 	window->priv->manager = manager;
 	g_signal_connect (manager, "add_widget", G_CALLBACK (add_widget), window);
@@ -1213,7 +1308,6 @@ sync_tab_icon (EphyTab *tab,
 
 	icon = ephy_tab_get_icon (tab);
 
-	gtk_window_set_icon (GTK_WINDOW (window), icon);
 	ephy_toolbar_set_favicon (priv->toolbar, icon);
 }
 
@@ -1271,14 +1365,6 @@ sync_tab_navigation (EphyTab *tab,
 		forward = TRUE;
 	}
 
-	action_group = window->priv->action_group;
-	action = gtk_action_group_get_action (action_group, "GoUp");
-	ephy_action_change_sensitivity_flags (action, SENS_FLAG_NAVIGATION, !up);
-	action = gtk_action_group_get_action (action_group, "GoBack");
-	ephy_action_change_sensitivity_flags (action, SENS_FLAG_NAVIGATION, !back);
-	action = gtk_action_group_get_action (action_group, "GoForward");
-	ephy_action_change_sensitivity_flags (action, SENS_FLAG_NAVIGATION, !forward);
-
 	ephy_toolbar_set_navigation_actions (window->priv->toolbar,
 					     back, forward, up);
 }
@@ -1292,7 +1378,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	char *state = NULL;
 	char *tooltip;
 	const char *stock_id = STOCK_LOCK_INSECURE;
-	gboolean show_lock = FALSE;
+	gboolean show_lock = FALSE, is_secure = FALSE;
 
 	if (window->priv->closing) return;
 
@@ -1329,6 +1415,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 			state = _("High");
 			stock_id = STOCK_LOCK_SECURE;
 			show_lock = TRUE;
+			is_secure = TRUE;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -1348,7 +1435,7 @@ sync_tab_security (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 	ephy_statusbar_set_security_state (EPHY_STATUSBAR (window->priv->statusbar),
 					   stock_id, tooltip);
 
-	ephy_toolbar_set_security_state (window->priv->toolbar,
+	ephy_toolbar_set_security_state (window->priv->toolbar, is_secure,
 					 show_lock, stock_id, tooltip);
 
 	if (window->priv->fullscreen_popup != NULL)
@@ -1461,7 +1548,8 @@ sync_tab_title (EphyTab *tab,
 
 	if (priv->closing) return;
 
-	gtk_window_set_title (GTK_WINDOW(window), ephy_tab_get_title (tab));
+	gtk_window_set_title (GTK_WINDOW(window),
+			      ephy_tab_get_title_composite (tab));
 }
 
 static void
@@ -1532,17 +1620,21 @@ sync_tab_zoom (EphyTab *tab, GParamSpec *pspec, EphyWindow *window)
 }
 
 static void
-network_status_changed (EphyEmbedSingle *single,
-			gboolean offline,
-			EphyWindow *window)
+sync_network_status (EphyEmbedSingle *single,
+		     GParamSpec *pspec,
+		     EphyWindow *window)
 {
+	EphyWindowPrivate *priv = window->priv;
 	GtkAction *action;
+	gboolean is_online;
 
-	action = gtk_action_group_get_action (window->priv->action_group,
+	is_online = ephy_embed_single_get_network_status (single);
+
+	action = gtk_action_group_get_action (priv->action_group,
 					      "FileWorkOffline");
 	g_signal_handlers_block_by_func
 		(action, G_CALLBACK (window_cmd_file_work_offline), window);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), offline);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), !is_online);
 	g_signal_handlers_unblock_by_func
 		(action, G_CALLBACK (window_cmd_file_work_offline), window);	
 }
@@ -1583,9 +1675,9 @@ idle_unref_context_event (EphyWindow *window)
 	return FALSE;
 }
 
-static void
-set_context_event (EphyWindow *window,
-		   EphyEmbedEvent *event)
+void
+_ephy_window_set_context_event (EphyWindow *window,
+				EphyEmbedEvent *event)
 {
 	EphyWindowPrivate *priv = window->priv;
 
@@ -1603,19 +1695,11 @@ set_context_event (EphyWindow *window,
 	priv->context_event = event != NULL ? g_object_ref (event) : NULL;
 }
 
-static void
-embed_popup_deactivate_cb (GtkWidget *popup,
-		          EphyWindow *window)
+void
+_ephy_window_unset_context_event (EphyWindow *window)
 {
 	EphyWindowPrivate *priv = window->priv;
 
-	LOG ("Deactivating popup menu");
-
-	enable_edit_actions_sensitivity (window);
-
-	g_signal_handlers_disconnect_by_func
-		(popup, G_CALLBACK (embed_popup_deactivate_cb), window);
-	
 	/* Unref the event from idle since we still need it
 	 * from the action callbacks which will run before idle.
 	 */
@@ -1626,6 +1710,20 @@ embed_popup_deactivate_cb (GtkWidget *popup,
 	}
 }
 
+static void
+embed_popup_deactivate_cb (GtkWidget *popup,
+			   EphyWindow *window)
+{
+	LOG ("Deactivating popup menu");
+
+	enable_edit_actions_sensitivity (window);
+
+	g_signal_handlers_disconnect_by_func
+		(popup, G_CALLBACK (embed_popup_deactivate_cb), window);
+
+	_ephy_window_unset_context_event (window);
+}
+
 static char *
 get_name_from_address_value (const GValue *value)
 {
@@ -1651,18 +1749,6 @@ update_popups_tooltips (EphyWindow *window, EphyEmbedEvent *event)
 	GtkAction *action;
 	char *tooltip, *name;
 
-	if (ephy_embed_event_has_property (event, "background_image"))
-	{
-		ephy_embed_event_get_property (event, "background_image", &value);
-
-		action = gtk_action_group_get_action (group, "SaveBackgroundAs");
-		name = get_name_from_address_value (value);
-		tooltip = g_strdup_printf (_("Save background image '%s'"), name);
-		g_object_set (action, "tooltip", tooltip, NULL);
-		g_free (name);
-		g_free (tooltip);
-	}
-
 	context = ephy_embed_event_get_context (event);
 
 	if (context & EPHY_EMBED_CONTEXT_IMAGE)
@@ -1747,7 +1833,7 @@ show_embed_popup (EphyWindow *window,
 	EphyEmbedEventContext context;
 	const char *popup;
 	const GValue *value;
-	gboolean framed, has_background, can_open_in_new;
+	gboolean framed, can_open_in_new;
 	GtkWidget *widget;
 	guint button;
 
@@ -1760,37 +1846,22 @@ show_embed_popup (EphyWindow *window,
 	ephy_embed_event_get_property (event, "framed_page", &value);
 	framed = g_value_get_int (value);
 
-	has_background = ephy_embed_event_has_property (event, "background_image");
 	can_open_in_new = ephy_embed_event_has_property (event, "link-has-web-scheme");
 
 	context = ephy_embed_event_get_context (event);
 
 	LOG ("show_embed_popup context %x", context);
 
-	if ((context & EPHY_EMBED_CONTEXT_EMAIL_LINK) &&
-	    (context & EPHY_EMBED_CONTEXT_IMAGE))
-	{
-		popup = "/EphyImageEmailLinkPopup";
-	}
-	else if (context & EPHY_EMBED_CONTEXT_EMAIL_LINK)
+	if (context & EPHY_EMBED_CONTEXT_EMAIL_LINK)
 	{
 		popup = "/EphyEmailLinkPopup";
 		update_edit_actions_sensitivity (window, TRUE);
 	}
-	else if ((context & EPHY_EMBED_CONTEXT_LINK) &&
-		 (context & EPHY_EMBED_CONTEXT_IMAGE))
-	{
-		popup = "/EphyImageLinkPopup";
-	}
 	else if (context & EPHY_EMBED_CONTEXT_LINK)
 	{
 		popup = "/EphyLinkPopup";
 		update_edit_actions_sensitivity (window, TRUE);
 	}
-	else if (context & EPHY_EMBED_CONTEXT_IMAGE)
-	{
-		popup = "/EphyImagePopup";
-	}
 	else if (context & EPHY_EMBED_CONTEXT_INPUT)
 	{
 		popup = "/EphyInputPopup";
@@ -1815,9 +1886,6 @@ show_embed_popup (EphyWindow *window,
 	g_return_if_fail (widget != NULL);
 
 	action_group = window->priv->popups_action_group;
-	action = gtk_action_group_get_action (action_group, "SaveBackgroundAs");
-	ephy_action_change_sensitivity_flags (action, SENS_FLAG_CONTEXT, !has_background);
-	gtk_action_set_visible (action, has_background);
 
 	action = gtk_action_group_get_action (action_group, "OpenLinkInNewWindow");
 	gtk_action_set_sensitive (action, can_open_in_new);
@@ -1825,12 +1893,15 @@ show_embed_popup (EphyWindow *window,
 	action = gtk_action_group_get_action (action_group, "OpenLinkInNewTab");
 	ephy_action_change_sensitivity_flags (action, SENS_FLAG_CONTEXT, !can_open_in_new);
 
-	set_context_event (window, event);
+	update_image_actions_visibility (window, context & EPHY_EMBED_CONTEXT_IMAGE);
+
+	_ephy_window_set_context_event (window, event);
 
 	g_signal_connect (widget, "deactivate",
 			  G_CALLBACK (embed_popup_deactivate_cb), window);
 
 	button = ephy_embed_event_get_button (event);
+
 	if (button == 0)
 	{
 		gtk_menu_popup (GTK_MENU (widget), NULL, NULL,
@@ -2087,8 +2158,8 @@ update_tabs_menu_sensitivity (EphyWindow *window)
 }
 
 static gboolean
-modal_alert_cb (EphyEmbed *embed,
-		EphyWindow *window)
+embed_modal_alert_cb (EphyEmbed *embed,
+		      EphyWindow *window)
 {
 	EphyWindowPrivate *priv = window->priv;
 	EphyTab *tab;
@@ -2115,6 +2186,83 @@ modal_alert_cb (EphyEmbed *embed,
 	return FALSE;
 }
 
+static gboolean
+idle_tab_remove_cb (EphyTab *tab)
+{
+	GtkWidget *toplevel;
+	EphyWindow *window;
+	EphyWindowPrivate *priv;
+	EphyNotebook *notebook;
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (tab));
+	if (!EPHY_IS_WINDOW (toplevel)) return FALSE; /* FIXME should this ever occur? */
+
+	window = EPHY_WINDOW (toplevel);
+	priv = window->priv;
+
+	if (priv->closing) return FALSE;
+
+	g_hash_table_remove (priv->tabs_to_remove, tab);
+
+	notebook = EPHY_NOTEBOOK (ephy_window_get_notebook (window));
+	ephy_notebook_remove_tab (notebook, tab);
+
+	/* don't run again */
+	return FALSE;
+}
+
+static void
+schedule_tab_close (EphyWindow *window,
+		    EphyEmbed *embed)
+{
+	EphyWindowPrivate *priv = window->priv;
+	EphyTab *tab;
+	guint id;
+
+	LOG ("scheduling close of embed %p in window %p", embed, window);
+
+	if (priv->closing) return;
+
+	tab = ephy_tab_for_embed (embed);
+	g_return_if_fail (tab != NULL);
+
+	if (g_hash_table_lookup (priv->tabs_to_remove, tab) != NULL) return;
+
+	/* do this on idle, because otherwise we'll crash in certain circumstances
+	* (see galeon bug #116256)
+	*/
+	id = g_idle_add_full (G_PRIORITY_HIGH_IDLE,
+			      (GSourceFunc) idle_tab_remove_cb,
+			      tab, NULL);
+
+	g_hash_table_insert (priv->tabs_to_remove, tab, GUINT_TO_POINTER (id));
+
+	/* don't wait until idle to hide the window */
+	if (g_hash_table_size (priv->tabs_to_remove) == priv->num_tabs)
+	{
+		gtk_widget_hide (GTK_WIDGET (window));
+	}
+}
+
+static gboolean
+embed_close_request_cb (EphyEmbed *embed,
+			EphyWindow *window)
+{
+	LOG ("embed_close_request_cb embed %p window %p", embed, window);
+
+	schedule_tab_close (window, embed);
+
+	/* handled */
+	return TRUE;
+}
+
+static void
+embed_destroy_browser_cb (EphyEmbed *embed,
+			  EphyWindow *window)
+{
+	g_return_if_reached ();
+}
+
 static gboolean
 show_notebook_popup_menu (GtkNotebook *notebook,
 			  EphyWindow *window,
@@ -2186,7 +2334,7 @@ tab_added_cb (EphyNotebook *notebook,
 	EphyExtension *manager;
 	EphyEmbed *embed;
 
-        g_return_if_fail (EPHY_IS_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	window->priv->num_tabs++;
 
@@ -2201,8 +2349,14 @@ tab_added_cb (EphyNotebook *notebook,
 	embed = ephy_tab_get_embed (tab);
 	g_return_if_fail (embed != NULL);
 
-	g_signal_connect_after (embed, "ge-modal-alert",
-				G_CALLBACK (modal_alert_cb), window);
+	g_signal_connect_object (embed, "close-request",
+				 G_CALLBACK (embed_close_request_cb),
+				 window, 0);
+	g_signal_connect_object (embed, "destroy-browser",
+				 G_CALLBACK (embed_destroy_browser_cb),
+				 window, 0);
+	g_signal_connect_object (embed, "ge-modal-alert",
+				 G_CALLBACK (embed_modal_alert_cb), window, G_CONNECT_AFTER);
 
 	/* Let the extensions attach themselves to the tab */
 	manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
@@ -2217,7 +2371,7 @@ tab_removed_cb (EphyNotebook *notebook,
 	EphyExtension *manager;
 	EphyEmbed *embed;
 
-        g_return_if_fail (EPHY_IS_TAB (tab));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
 	/* Let the extensions remove themselves from the tab */
 	manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
@@ -2241,7 +2395,11 @@ tab_removed_cb (EphyNotebook *notebook,
 	g_return_if_fail (embed != NULL);
 
 	g_signal_handlers_disconnect_by_func
-		(embed, G_CALLBACK (modal_alert_cb), window);
+		(embed, G_CALLBACK (embed_modal_alert_cb), window);
+	g_signal_handlers_disconnect_by_func
+			(embed, G_CALLBACK (embed_close_request_cb), window);
+	g_signal_handlers_disconnect_by_func
+			(embed, G_CALLBACK (embed_destroy_browser_cb), window);
 }
 
 static void
@@ -2271,24 +2429,28 @@ tabs_reordered_cb (EphyNotebook *notebook, EphyWindow *window)
 	update_tabs_menu_sensitivity (window);
 }
 
-static gboolean
-tab_delete_cb (EphyNotebook *notebook,
-	       EphyTab *tab,
-	       EphyWindow *window)
+static void
+tab_close_request_cb (EphyNotebook *notebook,
+		      EphyTab *tab,
+		      EphyWindow *window)
 {
-	g_return_val_if_fail (EPHY_IS_TAB (tab), FALSE);
+	EphyWindowPrivate *priv = window->priv;
+	EphyEmbed *embed;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_QUIT) &&
-	    gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook)) == 1)
+	    gtk_notebook_get_n_pages (priv->notebook) == 1)
 	{
-		return TRUE;
-	}
-	else if (ephy_embed_has_modified_forms (ephy_tab_get_embed (tab)))
-	{
-		return !confirm_close_with_modified_forms (window);
+		return;
 	}
 
-	return FALSE;
+	embed = ephy_tab_get_embed (tab);
+	g_return_if_fail (embed != NULL);
+
+	if (!ephy_embed_has_modified_forms (embed) ||
+	    confirm_close_with_modified_forms (window))
+	{
+		ephy_embed_close (embed);
+	}
 }
 
 static GtkNotebook *
@@ -2316,8 +2478,8 @@ setup_notebook (EphyWindow *window)
 			  G_CALLBACK (tab_detached_cb), NULL);
 	g_signal_connect (G_OBJECT (notebook), "tabs_reordered",
 			  G_CALLBACK (tabs_reordered_cb), window);
-	g_signal_connect (G_OBJECT (notebook), "tab_delete",
-			  G_CALLBACK (tab_delete_cb), window);
+	g_signal_connect (G_OBJECT (notebook), "tab_close_request",
+			  G_CALLBACK (tab_close_request_cb), window);
 
 	return notebook;
 }
@@ -2342,11 +2504,6 @@ ephy_window_set_chrome (EphyWindow *window, EphyEmbedChrome mask)
 		chrome_mask &= ~EPHY_EMBED_CHROME_STATUSBAR;
 	}
 
-	if (!eel_gconf_get_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR))
-	{
-		chrome_mask &= ~EPHY_EMBED_CHROME_BOOKMARKSBAR;
-	}
-
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_HIDE_MENUBAR))
 	{
 		chrome_mask &= ~EPHY_EMBED_CHROME_MENUBAR;
@@ -2376,6 +2533,12 @@ ephy_window_set_is_popup (EphyWindow *window,
 	g_object_notify (G_OBJECT (window), "is-popup");
 }
 
+static gboolean
+remove_true (void)
+{
+	return TRUE;
+}
+
 static void
 ephy_window_dispose (GObject *object)
 {
@@ -2396,6 +2559,7 @@ ephy_window_dispose (GObject *object)
 		/* Let the extensions detach themselves from the window */
 		manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
 		ephy_extension_detach_window (manager, window);
+		ephy_bookmarks_ui_detach_window (window);
 
 		/* Deactivate menus */
 		popups = gtk_ui_manager_get_toplevels (window->priv->manager, GTK_UI_MANAGER_POPUP);
@@ -2404,10 +2568,12 @@ ephy_window_dispose (GObject *object)
 	
 		single = ephy_embed_shell_get_embed_single (embed_shell);
 		g_signal_handlers_disconnect_by_func
-			(single, G_CALLBACK (network_status_changed), window);
+			(single, G_CALLBACK (sync_network_status), window);
 	
 		eel_gconf_notification_remove (priv->browse_with_caret_notifier_id);
 		eel_gconf_notification_remove (priv->allow_popups_notifier_id);
+		priv->browse_with_caret_notifier_id = 0;
+		priv->allow_popups_notifier_id = 0;
 
 		if (priv->idle_resize_handler != 0)
 		{
@@ -2415,8 +2581,7 @@ ephy_window_dispose (GObject *object)
 			priv->idle_resize_handler = 0;
 		}
 
-		g_object_unref (priv->fav_menu);
-		priv->fav_menu = NULL;
+		g_hash_table_foreach_remove (priv->tabs_to_remove, (GHRFunc) remove_true, NULL);
 
 		g_object_unref (priv->enc_menu);
 		priv->enc_menu = NULL;
@@ -2424,27 +2589,24 @@ ephy_window_dispose (GObject *object)
 		g_object_unref (priv->tabs_menu);
 		priv->tabs_menu = NULL;
 
-		g_object_unref (priv->bmk_menu);
-		priv->bmk_menu = NULL;
-	
 		if (priv->ppview_toolbar)
 		{
 			g_object_unref (priv->ppview_toolbar);
 			priv->ppview_toolbar = NULL;
 		}
-	
-		g_object_unref (priv->action_group);
+
 		priv->action_group = NULL;
+		priv->popups_action_group = NULL;
 
 		g_object_unref (priv->manager);
 		priv->manager = NULL;
 
-		set_context_event (window, NULL);
+		_ephy_window_set_context_event (window, NULL);
 	}
 
 	destroy_fullscreen_popup (window);
 
-        G_OBJECT_CLASS (parent_class)->dispose (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -2595,11 +2757,11 @@ ephy_window_class_init (EphyWindowClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
+	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->constructor = ephy_window_constructor;
 	object_class->dispose = ephy_window_dispose;
-        object_class->finalize = ephy_window_finalize;
+	object_class->finalize = ephy_window_finalize;
 	object_class->get_property = ephy_window_get_property;
 	object_class->set_property = ephy_window_set_property;
 
@@ -2684,14 +2846,6 @@ allow_popups_notifier (GConfClient *client,
 	}
 }
 
-static void
-action_request_forward_cb (GObject *toolbar,
-			   const char *name,
-                	   GObject *bookmarksbar)
-{
-	g_signal_emit_by_name (bookmarksbar, "action_request", name);
-}
-
 static EphyTab *
 ephy_window_open_link (EphyLink *link,
 		       const char *address,
@@ -2765,6 +2919,14 @@ find_toolbar_close_cb (EphyFindToolbar *toolbar,
 	gtk_widget_hide (GTK_WIDGET (toolbar));
 }
 
+static void
+cancel_handler (gpointer idptr)
+{
+	guint id = GPOINTER_TO_UINT (idptr);
+
+	g_source_remove (id);
+}
+
 static void
 ephy_window_init (EphyWindow *window)
 {
@@ -2780,6 +2942,9 @@ ephy_window_init (EphyWindow *window)
 
 	priv = window->priv = EPHY_WINDOW_GET_PRIVATE (window);
 
+	priv->tabs_to_remove = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+						      NULL, cancel_handler);
+
 	window->priv->chrome = EPHY_EMBED_CHROME_ALL;
 
 	ephy_gui_ensure_window_group (GTK_WINDOW (window));
@@ -2824,10 +2989,6 @@ ephy_window_init (EphyWindow *window)
 				  G_CALLBACK (ephy_link_open), window);
 	g_signal_connect_swapped (window->priv->toolbar, "exit-clicked",
 				  G_CALLBACK (exit_fullscreen_clicked_cb), window);
-	window->priv->bookmarksbar = ephy_bookmarksbar_new (window);
-	g_signal_connect_swapped (window->priv->bookmarksbar, "open-link",
-				  G_CALLBACK (ephy_link_open), window);
-
 	g_signal_connect_swapped (window->priv->toolbar, "activation-finished",
 				  G_CALLBACK (sync_chromes_visibility), window);
 
@@ -2838,30 +2999,13 @@ ephy_window_init (EphyWindow *window)
 	{
 		g_warning ("Could not merge epiphany-ui.xml: %s", error->message);
 		g_error_free (error);
-        }
+	}
 
 	/* Initialize the menus */
 	window->priv->tabs_menu = ephy_tabs_menu_new (window);
 	window->priv->enc_menu = ephy_encoding_menu_new (window);
-	window->priv->fav_menu = ephy_favorites_menu_new (window);
-	g_signal_connect_swapped (window->priv->fav_menu, "open-link",
-				  G_CALLBACK (ephy_link_open), window);
-	window->priv->bmk_menu = ephy_bookmarks_menu_new (window->priv->manager,
-							  BOOKMARKS_MENU_PATH);
-	g_signal_connect_swapped (window->priv->bmk_menu, "open-link",
-				  G_CALLBACK (ephy_link_open), window);
-
-	/* forward the toolbar's action_request signal to the bookmarks toolbar,
-	 * so the user can also have bookmarks on the normal toolbar
-	 */
-	g_signal_connect (window->priv->toolbar, "action_request",
-			  G_CALLBACK (action_request_forward_cb),
-			  window->priv->bookmarksbar);
 
 	/* Add the toolbars to the window */
-	gtk_box_pack_end (GTK_BOX (window->priv->menu_dock),
-			  window->priv->bookmarksbar,
-			  FALSE, FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (window->priv->menu_dock),
 			  GTK_WIDGET (window->priv->toolbar),
 			  FALSE, FALSE, 0);
@@ -2869,6 +3013,7 @@ ephy_window_init (EphyWindow *window)
 	/* Once the window is sufficiently created let the extensions attach to it */
 	manager = EPHY_EXTENSION (ephy_shell_get_extensions_manager (ephy_shell));
 	ephy_extension_attach_window (manager, window);
+	ephy_bookmarks_ui_attach_window (window);
 
 	/* We only set the model now after attaching the extensions, so that
 	 * extensions already have created their actions which may be on
@@ -2889,11 +3034,9 @@ ephy_window_init (EphyWindow *window)
 
 	/* network status */
 	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
-	network_status_changed (single,
-			        ephy_embed_single_get_offline_mode (single),
-				window);
-	g_signal_connect (single, "network-status",
-			  G_CALLBACK (network_status_changed), window);
+	sync_network_status (single, NULL, window);
+	g_signal_connect (single, "notify::network-status",
+			  G_CALLBACK (sync_network_status), window);
 
 	/* ensure the UI is updated */
 	gtk_ui_manager_ensure_update (window->priv->manager);
@@ -2922,9 +3065,18 @@ ephy_window_constructor (GType type,
 static void
 ephy_window_finalize (GObject *object)
 {
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+	EphyWindow *window = EPHY_WINDOW (object);
+	EphyWindowPrivate *priv = window->priv;
 
-	LOG ("Ephy Window finalized %p", object);
+	g_hash_table_destroy (priv->tabs_to_remove);
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+
+	LOG ("EphyWindow finalised %p", object);
+
+#ifdef ENABLE_PYTHON
+	ephy_python_schedule_gc ();
+#endif
 
 	g_object_unref (ephy_shell);
 }
@@ -3035,22 +3187,6 @@ ephy_window_get_toolbar (EphyWindow *window)
 	return GTK_WIDGET (window->priv->toolbar);
 }
 
-/**
- * ephy_window_get_bookmarksbar:
- * @window: an #EphyWindow
- *
- * Returns this window's bookmarks toolbar, which is an #EggEditableToolbar.
- *
- * Return value: an #EggEditableToolbar
- **/
-GtkWidget *
-ephy_window_get_bookmarksbar (EphyWindow *window)
-{
-	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
-
-	return GTK_WIDGET (window->priv->bookmarksbar);
-}
-
 /**
  * ephy_window_get_notebook:
  * @window: an #EphyWindow
@@ -3171,7 +3307,7 @@ real_get_active_tab (EphyWindow *window, int page_num)
  **/
 void
 ephy_window_remove_tab (EphyWindow *window,
-		        EphyTab *tab)
+			EphyTab *tab)
 {
 	EphyEmbed *embed;
 	gboolean modified;
@@ -3350,10 +3486,10 @@ ephy_window_set_zoom (EphyWindow *window,
 	EphyEmbed *embed;
 	float current_zoom = 1.0;
 
-        g_return_if_fail (EPHY_IS_WINDOW (window));
+	g_return_if_fail (EPHY_IS_WINDOW (window));
 
 	embed = ephy_window_get_active_embed (window);
-        g_return_if_fail (embed != NULL);
+	g_return_if_fail (embed != NULL);
 
 	current_zoom = ephy_embed_get_zoom (embed);
 
@@ -3379,8 +3515,6 @@ sync_prefs_with_chrome (EphyWindow *window)
 
 	if (window->priv->should_save_chrome)
 	{
-		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR,
-				       flags & EPHY_EMBED_CHROME_BOOKMARKSBAR);
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_TOOLBARS,
 				       flags & EPHY_EMBED_CHROME_TOOLBAR);
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_STATUSBAR,
@@ -3418,14 +3552,6 @@ ephy_window_view_toolbar_cb (GtkAction *action,
 				      EPHY_EMBED_CHROME_TOOLBAR);
 }
 
-static void
-ephy_window_view_bookmarksbar_cb (GtkAction *action,
-			          EphyWindow *window)
-{
-	sync_chrome_with_view_toggle (action, window,
-				      EPHY_EMBED_CHROME_BOOKMARKSBAR);
-}
-
 static void
 ephy_window_view_popup_windows_cb (GtkAction *action,
 				   EphyWindow *window)
