@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: pdm-dialog.c,v 1.53.4.2 2005/11/17 18:55:54 chpe Exp $
+ *  $Id: pdm-dialog.c,v 1.55 2005/10/22 22:11:05 jframeau Exp $
  */
 
 #include "config.h"
@@ -41,6 +41,7 @@
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtknotebook.h>
+#include <gtk/gtktogglebutton.h>
 #include <glib/gi18n.h>
 
 #include <time.h>
@@ -90,6 +91,7 @@ enum
 {
 	COL_PASSWORDS_HOST,
 	COL_PASSWORDS_USER,
+	COL_PASSWORDS_PASSWORD,
 	COL_PASSWORDS_DATA
 };
 
@@ -101,7 +103,8 @@ enum
 	PROP_COOKIES_REMOVE,
 	PROP_COOKIES_PROPERTIES,
 	PROP_PASSWORDS_TREEVIEW,
-	PROP_PASSWORDS_REMOVE
+	PROP_PASSWORDS_REMOVE,
+	PROP_PASSWORDS_SHOW
 };
 
 static const
@@ -110,11 +113,12 @@ EphyDialogProperty properties [] =
 	{ "pdm_dialog",			NULL, PT_NORMAL, 0 },
 	{ "pdm_notebook",		NULL, PT_NORMAL, 0 },
 
-	{ "cookies_treeview",		NULL, PT_NORMAL, 0 },
-	{ "cookies_remove_button",	NULL, PT_NORMAL, 0 },
-	{ "cookies_properties_button",	NULL, PT_NORMAL, 0 },
-	{ "passwords_treeview",		NULL, PT_NORMAL, 0 },
-	{ "passwords_remove_button",	NULL, PT_NORMAL, 0 },
+	{ "cookies_treeview",	   	NULL, PT_NORMAL, 0 },
+	{ "cookies_remove_button",     	NULL, PT_NORMAL, 0 },
+	{ "cookies_properties_button", 	NULL, PT_NORMAL, 0 },
+	{ "passwords_treeview",	       	NULL, PT_NORMAL, 0 },
+	{ "passwords_remove_button",   	NULL, PT_NORMAL, 0 },
+	{ "passwords_show_button",     	NULL, PT_NORMAL, 0 },
 
 	{ NULL }
 };
@@ -124,13 +128,11 @@ static void pdm_dialog_init		(PdmDialog *dialog);
 static void pdm_dialog_finalize		(GObject *object);
 
 /* Glade callbacks */
-void pdm_dialog_close_button_clicked_cb 		(GtkWidget *button,
+void pdm_dialog_close_button_clicked_cb 	  	(GtkWidget *button,
 							 PdmDialog *dialog);
 void pdm_dialog_cookies_properties_button_clicked_cb	(GtkWidget *button,
 							 PdmDialog *dialog);
-void pdm_dialog_cookies_treeview_selection_changed_cb	(GtkTreeSelection *selection,
-							 PdmDialog *dialog);
-void pdm_dialog_passwords_treeview_selection_changed_cb	(GtkTreeSelection *selection,
+void pdm_dialog_passwords_show_cb	 		(GtkWidget *checkbox,
 							 PdmDialog *dialog);
 void pdm_dialog_response_cb (GtkDialog *widget,
 			     int response,
@@ -397,7 +399,9 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (liststore),
+					      COL_COOKIES_HOST,
+					      GTK_SORT_ASCENDING);
 	info->model = GTK_TREE_MODEL (liststore);
 	g_object_unref (liststore);
 
@@ -573,11 +577,6 @@ pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 			 G_CALLBACK (cookie_deleted_cb), info->dialog);
 	g_signal_connect (manager, "cookies-cleared",
 			 G_CALLBACK (cookies_cleared_cb), info->dialog);
-
-	/* Now turn on sorting */
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
-					      COL_COOKIES_HOST,
-					      GTK_SORT_ASCENDING);
 }
 
 static void
@@ -640,7 +639,8 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 			(EPHY_DIALOG(dialog), properties[PROP_PASSWORDS_TREEVIEW].id));
 
 	/* set tree model */
-	liststore = gtk_list_store_new (3,
+	liststore = gtk_list_store_new (4,
+					G_TYPE_STRING,
 					G_TYPE_STRING,
 					G_TYPE_STRING,
 					EPHY_TYPE_PASSWORD_INFO);
@@ -648,7 +648,9 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_set_headers_visible (treeview, TRUE);
 	selection = gtk_tree_view_get_selection (treeview);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (liststore),
+					      COL_PASSWORDS_HOST,
+					      GTK_SORT_ASCENDING);
 	info->model = GTK_TREE_MODEL (liststore);
 	g_object_unref (liststore);
 
@@ -678,6 +680,20 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_USER);
 
+	gtk_tree_view_insert_column_with_attributes (treeview,
+						     COL_PASSWORDS_PASSWORD,
+						     _("User Password"),
+						     renderer,
+						     "text", COL_PASSWORDS_PASSWORD,
+						     NULL);
+	column = gtk_tree_view_get_column (treeview, COL_PASSWORDS_PASSWORD);
+	/* Hide this info by default */
+	gtk_tree_view_column_set_visible (column, FALSE);
+	gtk_tree_view_column_set_resizable (column, TRUE);
+	gtk_tree_view_column_set_reorderable (column, TRUE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sort_column_id (column, COL_PASSWORDS_PASSWORD);
+
 	info->treeview = treeview;
 
 	setup_action (info);
@@ -687,19 +703,12 @@ static void
 passwords_changed_cb (EphyPasswordManager *manager,
 		      PdmDialog *dialog)
 {
-	GtkTreeModel *model = dialog->priv->passwords->model;
-
 	LOG ("passwords changed");
 
 	/* since the callback doesn't carry any information about what
 	 * exactly has changed, we have to rebuild the list from scratch.
 	 */
-	gtk_list_store_clear (GTK_LIST_STORE (model));
-
-	/* And turn off sorting */
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
-					      GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
-					      GTK_SORT_ASCENDING);
+	gtk_list_store_clear (GTK_LIST_STORE (dialog->priv->passwords->model));
 
 	dialog->priv->passwords->fill (dialog->priv->passwords);
 }
@@ -731,10 +740,6 @@ pdm_dialog_fill_passwords_list (PdmActionInfo *info)
 	}
 
 	info->filled = TRUE;
-
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
-					      COL_PASSWORDS_HOST,
-					      GTK_SORT_ASCENDING);
 }
 
 static void
@@ -758,6 +763,7 @@ pdm_dialog_password_add (PdmActionInfo *info,
 			    &iter,
 			    COL_PASSWORDS_HOST, pinfo->host,
 			    COL_PASSWORDS_USER, pinfo->username,
+			    COL_PASSWORDS_PASSWORD, pinfo->password,
 			    -1);
 
 	g_value_init (&value, EPHY_TYPE_PASSWORD_INFO);
@@ -828,9 +834,6 @@ pdm_dialog_init (PdmDialog *dialog)
 				  properties[PROP_NOTEBOOK].id, &notebook,
 				  NULL);
 
-	ephy_gui_ensure_window_group (GTK_WINDOW (window));
-
-	gtk_window_set_role (GTK_WINDOW (window), "epiphany-cookie-manager");
 	gtk_window_set_icon_name (GTK_WINDOW (window), "web-browser");
 
 	/**
@@ -922,7 +925,8 @@ show_cookies_properties (PdmDialog *dialog,
 			       -1, -1, FALSE,
 			       EPHY_STATE_WINDOW_SAVE_SIZE | EPHY_STATE_WINDOW_SAVE_POSITION);
 	gtk_dialog_set_has_separator (GTK_DIALOG(gdialog), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(gdialog), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (gdialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (gdialog)->vbox), 2);
 
 	table = gtk_table_new (2, 4, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 5);
@@ -1048,16 +1052,37 @@ pdm_dialog_cookies_properties_button_clicked_cb (GtkWidget *button,
 	g_list_free (l);
 }
 
+void
+pdm_dialog_passwords_show_cb (GtkWidget *button,
+			      PdmDialog *dialog)
+{
+	GtkTreeView *treeview;
+	GtkTreeViewColumn *column;
+	gboolean active;
+
+	treeview = GTK_TREE_VIEW (ephy_dialog_get_control
+			(EPHY_DIALOG(dialog), properties[PROP_PASSWORDS_TREEVIEW].id));
+	column = gtk_tree_view_get_column (treeview, COL_PASSWORDS_PASSWORD);
+
+	active = gtk_toggle_button_get_active ((GTK_TOGGLE_BUTTON (button)));
+	
+	gtk_tree_view_column_set_visible (column, active);
+}
+
 void
 pdm_dialog_response_cb (GtkDialog *widget,
 			gint response,
 			PdmDialog *dialog)
 {
-	if (response == GTK_RESPONSE_HELP)
+	switch (response)
 	{
-		pdm_dialog_show_help (dialog);
-		return;
+		case GTK_RESPONSE_CLOSE:
+			g_object_unref (dialog);
+			break;
+		case GTK_RESPONSE_HELP:
+			pdm_dialog_show_help (dialog);
+			break;
+		default:
+			break;
 	}
-
-	g_object_unref (dialog);
 }
